jQuery(document).ready(function($){
	
	var paypal = {
			container: '.payment_method_' + braintree_paypal_vars.gateway_id,
			button_id: '#braintree_paypal_button',
			token_selector: '.bfwc-payment-method-token',
			nonce_selector: '.bfwc-nonce-value',
			device_data_selector: '.bfwc-device-data',
			init: function(){
				
				this.assign_parent_class();
				
				$(document.body).on('click', paypal.button_id, paypal.tokenize_method);
				
				$(document.body).on('click', '.bfwc-paypal-cancel', paypal.remove_method);
				
				//checkout functionality
				$(document.body).on('updated_checkout', this.check_paypal);
				$(document.body).on('checkout_error', this.checkout_error);
				$(paypal.container).closest('form').on('checkout_place_order_' + braintree_paypal_vars.gateway_id, this.woocommerce_form_submit);
				
				//other page functionality
				$(this.container).closest('form').on('woocommerce_form_submit_' + braintree_paypal_vars.gateway_id, this.woocommerce_form_submit);
				
				this.initialize_paypal();
				
			},
			assign_parent_class: function(){
				//ensure container has proper classes.
				$('input[name="payment_method"]').each(function(){
					
					var payment_gateway = $(this).val();
					
					if( !$(this).closest('li').hasClass('payment_method_' + payment_gateway)){
						
						$(this).closest('li').addClass('payment_method_' + payment_gateway);
					}
				})
			},
			initialize_paypal: function(){
				if(!$(paypal.container).length){
					return;
				}
				braintree.client.create({
					authorization: braintree_paypal_vars.client_token
				}, function(err, clientInstance){
					if(err){
						paypal.submit_error(err);
						return;
					}
					paypal.client = clientInstance;
					
					if(braintree_paypal_vars.advanced_fraud.enabled && !paypal.is_checkout_flow()){
						paypal.initialize_fraud_tools(clientInstance);
					}
					
					braintree.paypal.create({
						client: clientInstance
					}, function(err, paypalInstance){
						if(err){
							paypal.submit_error(err);
							return;
						}
						paypal.paypalInstance = paypalInstance;
					})
					
				})
			},
			check_paypal: function(e){
				
			},
			initialize_fraud_tools: function(client){
				braintree.dataCollector.create({
					client: client,
					kount: true
				}, function(err, dataCollectorInstance){
					if(err){
						paypal.submit_error(err);
						return;
					}
					$(paypal.container).find(paypal.device_data_selector).val(dataCollectorInstance.deviceData);
				})
			},
			tokenize_method: function(e){
				e.preventDefault();
				paypal.paypalInstance.tokenize(braintree_paypal_vars.options, function(err, payload){
					if(err){
						if(err.code === 'PAYPAL_POPUP_CLOSED'){
							return;
						}
						paypal.submit_error(err);
						return;
					}
					paypal.on_payment_method_recieved(payload);
				})
			},
			on_payment_method_recieved: function(response){
				paypal.payment_method_received = true;
				
				var container = $('#braintree-paypal-tokenized');
				
				var html = $(braintree_paypal_vars.html);
				
				$(html).find('.payment-method-description').text(response.details.email);
				
				container.append(html);
				
				$('.braintree-paypal-button').slideUp(400);
				
				container.slideDown(400);

				$(paypal.container).find(paypal.nonce_selector).val(response.nonce);
				
				if(braintree_paypal_vars.form.submit){
					$(paypal.container).closest('form').submit();
				}
			},
			has_payment_methods: function(){
				return $('#paypal_payment_methods').length > 0;
			},
			is_payment_method_selected: function(){
				if($(paypal.container).find(paypal.token_selector).length){
					if($(paypal.container).find(paypal.token_selector).val() !== ''){
						return true;
					}else{
						return false;
					}
				}else{
					return false;
				}
			},
			remove_method: function(){
				$(paypal.container).find(paypal.nonce_selector).val('');
				var container = $('#braintree-paypal-tokenized');
				paypal.payment_method_received = false;
				container.slideUp(400, function(){
					$(this).empty();
					$('.braintree-paypal-button').slideDown(400);
				})
			},
			submit_error: function(error){
				$(document.body).triggerHandler('bfwc_submit_error', {error: error, element: paypal.container});
			},
			checkout_error: function(){
				paypal.payment_method_received = false;
			},
			woocommerce_form_submit: function(e){
				if(paypal.is_payment_method_selected()){
					return true;
				}else{
					if(paypal.payment_method_received){
						return true;
					}else{
						return false;
					}
				}
			},
			is_checkout_flow: function(){
				return braintree_paypal_vars.options.flow === 'checkout';
			}
	}
	paypal.init();
})