<?php

use Braintree\WebhookNotification;

/**
 * WooCommerce admin functions.
 */

/**
 * Return an array of admin pages that are associated with the plugin.
 *
 * @return mixed
 */
function bfwc_admin_pages()
{
	global $bfwc_admin_pages;
	return apply_filters( 'braintree_wc_admin_pages', $bfwc_admin_pages );
}

function bfwc_add_submenu_page( $menu )
{
	global $bfwc_admin_pages;
	$bfwc_admin_pages [] = $menu [ 'menu_slug' ];
	$page = add_submenu_page( $menu [ 'parent_slug' ], $menu [ 'page_title' ], $menu [ 'menu_title' ], $menu [ 'capability' ], $menu [ 'menu_slug' ], $menu [ 'callback' ] );
	if ( isset( $menu [ 'load_page_callback' ] ) ) {
		add_action( 'load-' . $page, $menu [ 'load_page_callback' ] );
	}
}

/**
 * Return an array of UTC timezones
 */
function bwc_admin_get_timezones()
{
	$timezones = array ();
	$timezone_list = DateTimeZone::listAbbreviations( DateTimeZone::ALL );
	foreach ( $timezone_list as $key => $zones ) {
		foreach ( $zones as $timezone ) {
			try {
				$date_time_zone = new DateTimeZone( $timezone [ 'timezone_id' ] );
				$timezones [ $timezone [ 'timezone_id' ] ] = sprintf( '%s - UTC Offset: %s hrs', $timezone [ 'timezone_id' ], $date_time_zone->getOffset( new DateTime() ) / 3600 );
			} catch( Exception $e ) {
			}
		
		}
	}
	return $timezones;
}

function bwc_admin_webhooks()
{
	return apply_filters( 'bwc_admin_webhooks', array (
			WebhookNotification::SUBSCRIPTION_CANCELED => __( 'Subscription Cancelled', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_CHARGED_SUCCESSFULLY => __( 'Subscription Charged Successfully', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_CHARGED_UNSUCCESSFULLY => __( 'Subscription Charged Unsuccesfully', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_EXPIRED => __( 'Subscription Expired', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_TRIAL_ENDED => __( 'Subscription Trial Ended', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_WENT_ACTIVE => __( 'Subscription Went Active', 'braintree' ), 
			WebhookNotification::SUBSCRIPTION_WENT_PAST_DUE => __( 'Subscription Went Past Due', 'braintree' ), 
			// WebhookNotification::TRANSACTION_SETTLED => __( 'Transaction Settled', 'braintree' ),
			// WebhookNotification::TRANSACTION_SETTLEMENT_DECLINED => __( 'Transaction Settlement Declined', 'braintree' ),
			WebhookNotification::CHECK => __( 'Webhook Connection Check', 'braintree' ) 
	) );
}

/**
 * function that parses content between a start and end tag.
 *
 * @param string $content        	
 * @param string $start        	
 * @param string $end        	
 * @param bool $include_start_end        	
 */
function bwc_admin_parse_contents( $content, $start, $end, $include_start_end = true )
{
	// <subscription><add-ons></add-ons></subscription>
	$strpos = strpos( $content, $start ) + strlen( $start ); // $start = $subscription
	$strpos2 = strrpos( $content, $end ); // last occurance of $end
	$length = absint( $strpos2 - $strpos );
	$contents = substr( $content, $strpos, $length );
	return $include_start_end ? trim( sprintf( '%s%s%s', $start, $contents, $end ) ) : trim( $contents );
}

function bfwc_admin_get_template( $template, $args = array() )
{
	extract( $args );
	$file = bt_manager()->plugin_admin_path() . $template;
	if ( ! file_exists( $file ) ) {
		_doing_it_wrong( __FUNCTION__, sprintf( 'template file %s does not exist.', 'braintree' ), bt_manager()->version );
	}
	include $file;
}

/**
 * Return an array of WC order statuses.
 */
function bwc_admin_wc_get_order_statuses()
{
	$order_statuses = array (
			'wc-pending' => _x( 'Pending Payment', 'Order status', 'woocommerce' ), 
			'wc-processing' => _x( 'Processing', 'Order status', 'woocommerce' ), 
			'wc-on-hold' => _x( 'On Hold', 'Order status', 'woocommerce' ), 
			'wc-completed' => _x( 'Completed', 'Order status', 'woocommerce' ), 
			'wc-cancelled' => _x( 'Cancelled', 'Order status', 'woocommerce' ), 
			'wc-refunded' => _x( 'Refunded', 'Order status', 'woocommerce' ), 
			'wc-failed' => _x( 'Failed', 'Order status', 'woocommerce' ) 
	);
	return $order_statuses;
}

function bfwc_admin_get_subscription_plans()
{
	$key = sprintf( 'braintree_wc_%s_plans', bt_manager()->get_environment() );
	return get_transient( $key );
}

function bfwc_admin_status_name( $status )
{
	$statuses = array (
			'active' => __( 'Active', 'braintree' ), 
			'inactive' => __( 'Inactive', 'braintree' ), 
			'expired' => __( 'Expired', 'braintree' ), 
			'pending' => __( 'Pending', 'braintree' ) 
	);
	return isset( $statuses [ $status ] ) ? $statuses [ $status ] : $status;
}

function bfwc_admin_add_help_center_tab( $tabs )
{
	$tabs [] = array (
			'url' => 'https://support.paymentplugins.com', 
			'label' => __( 'Help Center', 'braintree' ), 
			'id' => '' 
	);
	return $tabs;
}
add_filter( 'bfwc_admin_header_tabs', 'bfwc_admin_add_help_center_tab' );

// If the merchant has a license, then update the plugin to the new pro version.
function bfwc_admin_update_pro( $plugins )
{
	global $bfwc_plugin_update;
	
	$plugin = 'woo-payment-gateway-pro/braintree-payments.php';
	$active_plugins = get_option( 'active_plugins' );
	$all_plugins = get_plugins();
	$license = bt_manager()->get_option( 'license' );
	
	if ( in_array( $plugin, $active_plugins ) || array_key_exists( $plugin, $all_plugins ) || empty( $license ) ) {
		$bfwc_plugin_update = false;
		return $plugins;
	}
	if ( $bfwc_plugin_update === false ) {
		return $plugins;
	}
	if ( $bfwc_plugin_update ) {
		$plugins->response [ $plugin ] = $bfwc_plugin_update;
		return $plugins;
	}
	
	$bfwc_plugin_update = ( object ) array (
			'new_version' => '2.5.4', 
			'package' => 'https://s3-us-west-1.amazonaws.com/woo-payment-gateway/woo-payment-gateway-pro.zip', 
			'url' => 'https://wordpress.org/plugins/woo-payment-gateway/' 
	);
	$plugins->response [ $plugin ] = $bfwc_plugin_update;
	
	add_action( 'upgrader_process_complete', 'bfwc_admin_upgrade_to_pro_success', 10, 2 );
	
	return $plugins;
}
add_filter( 'pre_set_site_transient_update_plugins', 'bfwc_admin_update_pro' );

function bfwc_admin_upgrade_to_pro_success( $upgrader, $data )
{
	$plugin = 'woo-payment-gateway-pro/braintree-payments.php';
	
	// deactivate the plugin.
	if ( isset( $data [ 'plugin' ] ) && $data [ 'plugin' ] === $plugin ) {
		deactivate_plugins( 'woo-payment-gateway/braintree-payments.php' );
		
		// unset so that the pro plugin will go through the updates file on admin_init
		$version = get_option( 'braintree_for_woocommerce_version' );
		$version [ 'currentVersion' ] = '2.5.3';
		unset( $version [ 'previousVersions' ] [ '2.5.4' ] );
		update_option( 'braintree_for_woocommerce_version', $version );
	}
}