<?php

class Braintree_Gateway_Donation_Settings extends Braintree_Gateway_Settings_API
{
	
	/**
	 *
	 * @var Braintree_Gateway_Donation_Modal_Settings
	 */
	public $modal_settings;
	
	public $custom_form_settings;
	
	public $merchant_account_settings;
	
	public $loader_settings;

	public function __construct()
	{
		$this->page = 'braintree-gateway-settings';
		$this->id = 'donation_settings';
		$this->tab = 'donation-settings';
		$this->label = __( 'Donations', 'braintree' );
		$this->title = array ( 
				'title' => __( 'Donation Settings', 'braintree' ), 
				'description' => __( 'On this page you can configure your donation settings. This plugin allows you to select the donation 
						form and the types of fields that will appear on your donation form.
						<p>To add a donation section on a page use shortcode <strong>[braintree_donations]</strong>. You can add a dropdown for the donation amount if desired. <strong>[braintree_donations 1="2" 2="5" 3="10"]</strong></p>
						<p>To add a recurring donation section on a page, use shortcode <strong>[braintree_recurring_donation 1="my_subscription_plan"]</strong>. Each plan your add will be added to a dropdown of selectable recurring donations.</p>', 'braintree' ) 
		);
		add_filter( 'braintree_settings_localized_variables', array ( 
				$this, 
				'localize_vars' 
		) );
		
		include_once 'class-donation-modal-settings.php';
		include_once 'class-donation-custom-form-settings.php';
		//include_once 'class-donation-merchant-account-settings.php';
		include_once 'class-donation-loader-settings.php';
		$this->modal_settings = new Braintree_Gateway_Donation_Modal_Settings();
		$this->custom_form_settings = new Braintree_Gateway_Donation_Custom_Form_Settings();
		//$this->merchant_account_settings = new Braintree_Gateway_Donation_Merchant_Account_Settings();
		$this->loader_settings = new Braintree_Gateway_Donation_Loader_Settings();
		parent::__construct();
	}

	public function localize_vars( $vars )
	{
		$keys = array ( 
				'donation_payment_methods', 
				'donation_fields' 
		);
		foreach ( $keys as $key ) {
			$field_key = $this->get_field_key_name( $key );
			$vars[ 'keys' ][ $field_key ] = array ( 
					'options' => $this->settings()[ $key ][ 'options' ], 
					'html' => '<div class="chip">%title%<input type="hidden" name="' . $field_key . '[%name%]" value=""><i class="remove-settings-chip close material-icons">close</i></div>', 
					'toast' => __( 'Field %s has already been added.', 'braintree' ) 
			);
		}
		return $vars;
	}

	public function settings()
	{
		return array ( 
				'donation_tax_exempt' => array ( 
						'title' => __( 'Tax Exempt', 'braintree' ), 
						'type' => 'checkbox', 
						'default' => '', 
						'value' => 'yes', 
						'class' => 'filled-in', 
						'tool_tip' => true, 
						'description' => __( 'If set to true, then the transaction will be passed to Braintree as tax exempt.', 'braintree' ) 
				), 
				'authorize_donation' => array ( 
						'type' => 'checkbox', 
						'title' => __( 'Authorize Transaction', 'braintree' ), 
						'class' => 'filled-in', 
						'default' => '', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, donation transactions will be authorized but not submitted for settlement. You can capture authorized charges via the Donations admin page.', 'braintree' ) 
				), 
				'donation_form_type' => array ( 
						'type' => 'select', 
						'title' => __( 'Payment Form Type', 'braintree' ), 
						'default' => 'dropin', 
						'value' => '', 
						'options' => array ( 
								'dropin' => __( 'Dropin Form', 'braintree' ), 
								'custom' => __( 'Custom Form', 'braintree' ) 
						), 
						'tool_tip' => true, 
						'description' => __( 'You can select if you want your donation payment form to be the Braintree dropin form or a custom form.', 'braintree' ) 
				), 
				'donation_custom_form_settings' => array ( 
						'type' => 'custom', 
						'title' => __( 'Custom Form Settings', 'braintree' ), 
						'function' => array ( 
								$this->custom_form_settings, 
								'output_settings' 
						), 
						'attributes' => array ( 
								'data-parent-setting' => "{$this->get_field_key_name('donation_form_type')}", 
								'data-show-if' => 'custom' 
						), 
						'tool_tip' => true, 
						'description' => __( 'If you have selected to use a custom form, this setting allows you to customize your custom form.', 'braintree' ) 
				), 
				'donation_loader' => array ( 
						'type' => 'custom', 
						'title' => __( 'Loader Settings', 'braintree' ), 
						'function' => array ( 
								$this->loader_settings, 
								'output_settings' 
						), 
						'tool_tip' => true, 
						'description' => __( 'The donation loader displays as an overlay over the donation form when the payment is being processed.
                                You can customize the styling and message that dispalys to your donors.', 'braintree' ) 
				), 
				'donation_form_layout' => array ( 
						'title' => __( 'Form Layout', 'braintree' ), 
						'type' => 'select', 
						'options' => array ( 
								'modal' => __( 'Modal', 'braintree' ), 
								'inline' => __( 'Inline', 'braintree' ) 
						), 
						'default' => 'modal', 
						'title' => __( 'Form Layout', 'braintree' ), 
						'class' => 'filled-in', 
						'tool_tip' => true, 
						'description' => __( 'The form layout has two options. If modal is selected, the form will appear as a popup when the donator
						clicks the donation button. If inline is selected, the donation form will appear inside the html of the page.', 'braintree' ) 
				), 
				'donation_modal_options' => array ( 
						'type' => 'custom', 
						'title' => __( 'Donation Modal Options', 'braintree' ), 
						'default' => '', 
						'value' => '', 
						'function' => array ( 
								$this->modal_settings, 
								'output_settings' 
						), 
						'attributes' => array ( 
								'data-parent-setting' => "{$this->get_field_key_name('donation_form_layout')}", 
								'data-show-if' => 'modal' 
						), 
						'tool_tip' => true, 
						'description' => __( 'These settings pertain to all of the modal options.', 'braintree' ) 
				), 
				'donation_button_text' => array ( 
						'type' => 'text', 
						'value' => 'Donate', 
						'default' => __( 'Donate', 'braintree' ), 
						'title' => __( 'Button Text', 'braintree' ), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'You can customize the text that appears on the donation button by entering the text here.', 'braintree' ) 
				), 
				'donation_button_background' => array ( 
						'type' => 'text', 
						'value' => '#61D395', 
						'default' => '#61D395', 
						'title' => __( 'Background Color', 'braintree' ), 
						'class' => 'color-option', 
						'tool_tip' => true, 
						'description' => __( 'You can customize the background color of the donation button by selecting a color from the color picker.', 'braintree' ) 
				), 
				'donation_button_border' => array ( 
						'type' => 'text', 
						'value' => '#61D395', 
						'default' => '#61D395', 
						'title' => __( 'Border Color', 'braintree' ), 
						'class' => 'color-option', 
						'tool_tip' => true, 
						'description' => __( 'You can customize the border color of the donation button by selecting a color from the color picker.', 'braintree' ) 
				), 
				'donation_button_text_color' => array ( 
						'type' => 'text', 
						'value' => '#61D395', 
						'default' => '#ffffff', 
						'title' => __( 'Text Color', 'braintree' ), 
						'class' => 'color-option', 
						'tool_tip' => true, 
						'description' => __( 'You can customize the text color of the button.', 'braintree' ) 
				), 
				'donation_fields' => array ( 
						'title' => __( 'Donation Fields', 'braintree' ), 
						'type' => 'multiselect', 
						'default' => array (), 
						'options' => array ( 
								'billing_first_name' => __( 'First Name', 'braintree' ), 
								'billing_last_name' => __( 'Last Name', 'braintree' ), 
								'billing_company' => __( 'Billing Company', 'braintre' ), 
								'billing_address_1' => __( 'Billing Address', 'braintree' ), 
								'billing_address_2' => __( 'Billing Address2', 'braintree' ), 
								'billing_country' => __( 'Billing Country', 'braintree' ), 
								'billing_city' => __( 'Billing City', 'braintree' ), 
								'billing_state' => __( 'Billing State', 'braintree' ), 
								'billing_postalcode' => __( 'Postal Code', 'braintree' ), 
								'email_address' => __( 'Email Address', 'braintree' ), 
								'donation_message' => __( 'Donation Message', 'braintree' ) 
						), 
						'tool_tip' => true, 
						'description' => __( 'This settings allows you to set each of the donation fields that will appear on your donation form.', 'braintree' ) 
				),
                /* 'donation_merchant_accounts' => array(
                        'type' => 'custom',
                        'title' => __('Merchant Account ID\'s', 'braintree'),
                        'function' => array(
                                $this->merchant_account_settings,
                                'output_settings'
                        ),
                        'class' => '',
                        'tool_tip' => true,
                        'description' => __(
                                'You can configure all of the different currencies that you want to accept donations in.', 
                                'braintree')
                ), */
                'donation_production_merchant_account_id' => array ( 
						'type' => 'text', 
						'value' => '', 
						'default' => '', 
						'title' => __( 'Production Merchant Account Id', 'braintree' ), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The merchant account ID determines the settlement currency of your donations. If left blank, the transcation will be settled using your default Braintree merchant account.', 'braintree' ) 
				),
				'donation_sandbox_merchant_account_id' => array(
						'type' => 'text',
						'value' => '',
						'default' => '',
						'title' => __( 'Sandbox Merchant Account Id', 'braintree' ),
						'class' => '',
						'tool_tip' => true,
						'description' => __( 'The merchant account ID determines the settlement currency of your donations. If left blank, the transcation will be settled using your default Braintree merchant account.', 'braintree' )
						
				),
				'donation_currency' => array ( 
						'type' => 'select', 
						'title' => __( 'Donation Currency', 'braintree' ), 
						'options' => braintree_get_currencies(), 
						'default' => 'USD', 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'You can set the currency that will display on the amount field. Ensure this currency matches the currency for your Braintree Account. If you have set a merchant account for donations, the donation currency will automatically use that currency.', 'braintree' ) 
				), 
				'donation_success_url' => array ( 
						'type' => 'text', 
						'value' => '', 
						'default' => '', 
						'title' => __( 'Success URL', 'braintree' ), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'Enter the url of the page/site you would like the customer to be redirected to after the donation. This page typically has a message thanking the donor and informing them about the donation.', 'braintree' ) 
				), 
				'donation_payment_methods' => array ( 
						'title' => __( 'Accepted Payment Methods', 'braintree' ), 
						'type' => 'multiselect', 
						'default' => array ( 
								'amex', 
								'discover', 
								'visa', 
								'master_card' 
						), 
						'options' => array ( 
								'amex' => __( 'Amex', 'braintree' ), 
								'china_union_pay' => __( 'China Union Pay', 'braintree' ), 
								'diners_club_international' => __( 'Diners', 'braintree' ), 
								'discover' => __( 'Discover', 'braintree' ), 
								'jcb' => __( 'JCB', 'braintree' ), 
								'maestro' => __( 'Maestro', 'braintree' ), 
								'master_card' => __( 'MasterCard', 'branintree' ), 
								'solo' => __( 'Solo', 'braintree' ), 
								'switch_type' => __( 'Switch', 'braintree' ), 
								'visa' => __( 'Visa', 'braintree' ), 
								'paypal' => __( 'PayPal', 'braintree' ) 
						), 
						'value' => '', 
						'tool_tip' => true, 
						'description' => __( 'These payment methods will appear as icons aboove the payment form.', 'braintree' ) 
				) 
		);
	}
}
new Braintree_Gateway_Donation_Settings();