<?php

class Braintree_Gateway_Merchant_Account_Settings extends Braintree_Gateway_Settings_API
{

	public function __construct()
	{
		$this->page = 'braintree-gateway-settings';
		$this->id = 'checkout_settings';
		$this->tab = 'checkout-settings';
		$this->title = array (
				'title' => __( 'Merchant Accounts', 'braintree' ), 
				'description' => __( 'Your merchant accounts determine the currency that your transactions are settled in. 
                       If you use a currency switcher, then you can add all of the Braintree merchant accounts for the currencies that you accept on your site. You can configure your Sandbox and Production merchant accounts.', 'braintree' ) 
		);
		add_filter( 'braintree_gateway_default_settings', array (
				$this, 
				'get_default_settings' 
		) );
		add_action( "braintree_gateway_{$this->tab}_save_settings", array (
				$this, 
				'save' 
		) );
		add_filter( 'braintree_settings_localized_variables', array (
				$this, 
				'localize_vars' 
		) );
		add_filter( 'braintree_gateway_validate_woocommerce_braintree_production_merchant_account_id', array (
				$this, 
				'validate_merchant_accounts' 
		), 10, 2 );
		add_filter( 'braintree_gateway_validate_woocommerce_braintree_sandbox_merchant_account_id', array (
				$this, 
				'validate_merchant_accounts' 
		), 10, 2 );
	}

	public function localize_vars( $vars )
	{
		$vars [ 'merchant_accounts' ] = array (
				'sandbox' => array (
						'text' => __( 'Merchant Account (%s)', 'braintree' ), 
						'html' => '<div class="col s8"><label>' . __( 'Sandbox Merchant Account (%currency%)', 'braintree' ) . '</label><input type="text" name="%name%" value="" placeholder="' . __( 'Enter Merchant Account', 'braintree' ) . '"/><i class="material-icons trash-merchant-account">delete</i></div>' 
				), 
				'production' => array (
						'text' => __( 'Merchant Account (%s)', 'braintree' ), 
						'html' => '<div class="col s8"><label>' . __( 'Production Merchant Account (%currency%)', 'braintree' ) . '</label><input type="text" name="%name%" value="" placeholder="' . __( 'Enter Merchant Account', 'braintree' ) . '"/><i class="material-icons trash-merchant-account">delete</i></div>' 
				), 
				'messages' => array (
						'no_currency' => __( 'Please select a currency from the dropdown so you can add your merchant account.', 'braintree' ), 
						'currency_exists' => __( 'You already have a merchant account configured for currency %currency%', 'braintree' ) 
				) 
		);
		return $vars;
	}

	public function settings()
	{
		return array (
				'woocommerce_braintree_production_merchant_account_id' => array (
						'type' => 'custom', 
						'title' => __( 'Production Merchant Accounts', 'braintree' ), 
						'save' => true, 
						'default' => array (), 
						'function' => array (
								$this, 
								'output_merchant_account_settings' 
						), 
						'attributes' => array (
								'environment' => 'production', 
								'data-setting' => $this->get_field_key_name( 'woocommerce_braintree_production_merchant_account_id' ) . '[%currency%]' 
						), 
						'label' => __( 'Production Merchant Account (%s)', 'braintree' ), 
						'tool_tip' => true, 
						'description' => __( 'Your merchant accounts determine the settlement currency of your transactions. This settings is optional but if you use a currency switcher it\'s
                                recommended that you configure your merchant accounts.', 'braintree' ) 
				), 
				'woocommerce_braintree_sandbox_merchant_account_id' => array (
						'type' => 'custom', 
						'title' => __( 'Sandbox Merchant Accounts', 'braintree' ), 
						'save' => true, 
						'default' => array (), 
						'function' => array (
								$this, 
								'output_merchant_account_settings' 
						), 
						'attributes' => array (
								'environment' => 'sandbox', 
								'data-setting' => $this->get_field_key_name( 'woocommerce_braintree_sandbox_merchant_account_id' ) . '[%currency%]' 
						), 
						'label' => __( 'Sandbox Merchant Account (%s)', 'braintree' ), 
						'tool_tip' => true, 
						'description' => __( 'Your merchant accounts determine the settlement currency of your transactions. This settings is optional but if you use a currency switcher it\'s
                                recommended that you configure your merchant accounts.', 'braintree' ) 
				) 
		);
	}

	public function output_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'html-helpers/settings-modal.php';
	}

	public function output_merchant_account_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'views/merchant-accounts.php';
	}

	public function validate_merchant_accounts( $value, $key )
	{
		if ( empty( $value ) ) { // array of merchant accounts is empty so exit.
			return $value;
		}
		$environment = strpos( $key, 'production' ) ? 'production' : 'sandbox';
		if ( $environment === 'production' ) {
			bt_manager()->add_admin_notice( 'error', __( 'You cannot add production merchant accounts while your license status is not active.', 'braintree' ) );
			return false;
		
		}
		// compare values, maybe an API call isn't needed if the vales haven't changed.
		$old_value = bt_manager()->get_option( $key );
		
		if ( $old_value == $value ) {
			return $value;
		}
		
		foreach ( $value as $currency => $account ) {
			if ( empty( $account ) ) {
				bt_manager()->add_admin_notice( 'error', __( 'Values for merchant accounts cannot be blank', 'braintree' ) );
				return false;
			}
			Braintree_Configuration::environment( $environment );
			Braintree_Configuration::merchantId( bt_manager()->get_option( "{$environment}_merchant_id" ) );
			Braintree_Configuration::privateKey( bt_manager()->get_option( "{$environment}_private_key" ) );
			Braintree_Configuration::publicKey( bt_manager()->get_option( "{$environment}_public_key" ) );
			try {
				$merchant_account = Braintree_MerchantAccount::find( $account );
			} catch( Braintree\Exception\NotFound $e ) {
				bt_manager()->add_admin_notice( 'error', sprintf( __( 'Merchant account %s could not be found in your %s environment.', 'braintree' ), $account, $environment ) );
				return false;
			} catch( \Braintree\Exception $e ) {
				bt_manager()->add_admin_notice( 'error', sprintf( __( 'There was an error retrieving merchant account %s. Reason: %s', 'braintree' ), $account, $e->getMessage() ) );
				return false;
			}
		}
		return $value;
	}
}