<?php

use Braintree\PaymentInstrumentType;
class Braintree_Gateway_PayPal_Settings extends Braintree_Gateway_Settings_API
{

	public function __construct()
	{
		$this->tab = 'checkout-settings';
		$this->page = 'braintree-gateway-settings';
		$this->id = 'checkout_settings';
		$this->title = array (
				'title' => __( 'PayPal Settings', 'braintree' ), 
				'description' => __( 'These options allow you to customize the PayPal Gateway.', 'braintree' ) 
		);
		add_filter( 'braintree_gateway_default_settings', array (
				$this, 
				'get_default_settings' 
		) );
		add_action( "braintree_gateway_{$this->tab}_save_settings", array (
				$this, 
				'save' 
		) );
	}

	public function output_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'html-helpers/settings-modal.php';
	}

	public function output_paypal_buttons( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'views/paypal-buttons.php';
	}

	public function settings()
	{
		return array (
				'enable_paypal' => array (
						'type' => 'checkbox', 
						'value' => 'yes', 
						'default' => 'yes', 
						'title' => __( 'PayPal Enabled', 'braintree' ), 
						'class' => 'filled-in', 
						'tool_tip' => true, 
						'description' => __( 'If you have enabled custom forms and want to use PayPal as a payment option you must select this option. PayPal will be treated as a separate gateway. You must ensure that you have linked your PayPal and Braintree accounts.', 'braintree' ) 
				), 
				'paypal_gateway_title' => array (
						'type' => 'text', 
						'value' => '', 
						'default' => __( 'PayPal', 'braintree' ), 
						'title' => __( 'PayPal Title', 'braintree' ), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'The value here determines the text that appears on the checkout page next to the PayPal gateway.', 'braintree' ) 
				), 
				'paypal_checkout_flow' => array (
						'type' => 'select', 
						'value' => '', 
						'default' => 'checkout', 
						'title' => __( 'Checkout With PayPal', 'braintree' ), 
						'options' => array (
								'checkout' => __( 'Checkout Flow', 'braintree' ), 
								'vault' => __( 'Vault Flow', 'braintree' ) 
						), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'This options allows you to set the checkout flow for PayPal. Vault flow is a shorter version of the checkout flow and will require your customers to grant your merchant site authorization.
						The checkout flow is the standard checkout options for PayPal. If you are using subscriptions, the vault flow will be used regardless of this setting as subscriptions require a billing agreement which
						the vault flow grants.', 'braintree' ) 
				), 
				'paypal_display_name' => array (
						'type' => 'text', 
						'default' => get_option( 'blogname' ), 
						'title' => __( 'PayPal Display Name', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'tool_tip' => true, 
						'description' => __( 'This is the name that will appear on the PayPal checkout screen if the customer chooses to pay with PayPal.', 'braintree' ) 
				), 
				'paypal_submit_form' => array (
						'type' => 'checkbox', 
						'title' => __( 'Submit Form Automatically', 'braintree' ), 
						'default' => 'yes', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, the order will be placed once the PayPal payment method is selected by the customer. If not enabled, the customer will still need to click the Place Order button to process the order.', 'braintree' ) 
				), 
				'paypal_format' => array (
						'title' => __( 'PayPal Method Display', 'braintree' ), 
						'type' => 'select', 
						'options' => $this->get_paypal_options(), 
						'value' => '', 
						'default' => 'paypal_and_email', 
						'tool_tip' => true, 
						'description' => __( 'This option allows you to customize how the PayPal method will display for your customers on orders, payment methods page, etc.' ) 
				), 
				'paypal_button_design' => array (
						'type' => 'select', 
						'title' => __( 'PayPal Button Design', 'braintree' ), 
						'default' => 0, 
						'options' => $this->get_paypal_button_options(), 
						'class' => '', 
						'value' => '', 
						'default' => 2, 
						'tool_tip' => true, 
						'description' => __( 'This option allows you to select the PayPal button design that you want to be displayed on the checkout page. If you want to
						create your own PayPal button, follow the tutorials.', 'braintree' ) 
				) 
		);
	}

	public function get_paypal_options()
	{
		$patterns = braintree_get_payment_method_formats() [ PaymentInstrumentType::PAYPAL_ACCOUNT ];
		$formats = array ();
		foreach ( $patterns as $k => $v ) {
			$formats [ $k ] = $v [ 'example' ];
		}
		return $formats;
	}

	/**
	 * Return an array of PayPal button options.
	 *
	 * @return array
	 */
	public function get_paypal_button_options()
	{
		$buttons = braintree_get_paypal_buttons();
		$options = array ();
		foreach ( $buttons as $index => $button ) {
			$options [ $index ] = $button [ 'name' ];
		}
		return $options;
	}
}