<?php

class Braintree_Gateway_Subscription_Settings extends Braintree_Gateway_Settings_API
{

	public function __construct()
	{
		$this->page = 'braintree-gateway-settings';
		$this->id = 'subscription_settings';
		$this->tab = 'subscription-settings';
		$this->label = __( 'Subscriptions', 'braintree' );
		$this->title = array (
				'title' => __( 'Subscription Settings', 'braintree' ), 
				'description' => __( 'If you do not have the WooCommerce Subscriptions plugin then you can use this plugin\'s subscription functionality.', 'braintree' ) 
		);
		add_filter( 'braintree_settings_localized_variables', array (
				$this, 
				'localize_vars' 
		) );
		
		add_action( 'init', array (
				$this, 
				'maybe_set_test_mode' 
		), - 900 );
		
		parent::__construct();
	}

	public function localize_vars( $vars )
	{
		$vars [ 'subscriptions' ] = array (
				'wcs_active' => bt_manager()->is_woocommerce_subscriptions_active() 
		);
		return $vars;
	}

	public function settings()
	{
		return array (
				'subscription_gateway_timezone' => array (
						'type' => 'select', 
						'title' => __( 'Gateway Timezone', 'braintree' ), 
						'options' => bwc_admin_get_timezones(), 
						'default' => 'America/Chicago', 
						'tool_tip' => true, 
						'description' => __( 'This is the timezone of your Braintree gateway. This value is used to calculate the Braintree subscription start dates.' ), 
						'helper' => array (
								'enabled' => true, 
								'title' => __( 'Gateway Timezone', 'braintree' ), 
								'description' => __( 'The gateway timezone is the timezone that your subscriptions are created in. To locate your gateay timezone, login to the Braintree control panel and navigate to the processing page and scroll to the bottom.', 'braintree' ), 
								'type' => 'img', 
								'url' => 'https://wordpress.paymentplugins.com/woo-payment-gateway/assets/subscription_gateway_timezone.png' 
						) 
				), 
				'woocommerce_subscriptions_prefix' => array (
						'type' => 'text', 
						'value' => '', 
						'default' => '', 
						'title' => __( 'Subscription Prefix', 'braintree' ), 
						'class' => '', 
						'tool_tip' => true, 
						'description' => __( 'If you would like the subscription order ID to contain an order prefix, you can add one here. If left blank, the order id within Braintree
						will match the WooCommerce order id.', 'braintree' ) 
				), 
				'braintree_subscription_combine_same_products' => array (
						'type' => 'checkbox', 
						'title' => __( 'Combine Subscriptions', 'braintree' ), 
						'default' => '', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, same products will be combined into one subscription instead of multiple subscriptions. For example, if a customer adds
								two of the same type of subscription, they will combined into one since they share the same billing schedule. Products with different currencies will not be combined.', 'braintree' ) 
				), 
				'wcs_enable_test_mode' => array (
						'type' => 'checkbox', 
						'title' => __( 'Enable Test Mode', 'braintree' ), 
						'default' => 'false', 
						'value' => 'yes', 
						'tool_tip' => true, 
						'description' => __( 'If enabled, you will be able to test WooCommerce Subscriptions using the action scheduler.', 'braintree' ) 
				) 
		);
	}

	public function maybe_set_test_mode()
	{
		if ( is_admin() && bt_manager()->is_active( 'wcs_enable_test_mode' ) ) {
			if ( defined( 'WP_DEBUG' ) && WP_DEBUG === false ) {
				add_filter( 'action_scheduler_post_type_args', __CLASS__ . '::action_scheduler_post_type_args' );
			}
		}
	}

	public static function action_scheduler_post_type_args( $args )
	{
		return array_merge( $args, array (
				'show_ui' => true, 
				'show_in_menu' => 'tools.php', 
				'show_in_admin_bar' => false 
		) );
	}
}
new Braintree_Gateway_Subscription_Settings();