<?php
use Braintree\WebhookNotification;
/**
 *
 * @author Payment Plugins
 * @copyright 2016
 */
class Braintree_Gateway_WCS_Controller
{

	public function __construct()
	{
		add_action( 'bfwc_validate_notification', array ( 
				$this, 
				'validate_webhook' 
		) );
		
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_CANCELED . '_subscription_controller', array ( 
				$this, 
				'subscription_cancelled' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_CHARGED_SUCCESSFULLY . '_subscription_controller', array ( 
				$this, 
				'charged_successfully' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_CHARGED_UNSUCCESSFULLY . '_subscription_controller', array ( 
				$this, 
				'charged_unsuccessfully' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_EXPIRED . '_subscription_controller', array ( 
				$this, 
				'subscription_expired' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_TRIAL_ENDED . '_subscription_controller', array ( 
				$this, 
				'subscription_trail_ended' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_WENT_ACTIVE . '_subscription_controller', array ( 
				$this, 
				'subscription_went_active' 
		), 10, 3 );
		add_action( 'bfwc_' . WebhookNotification::SUBSCRIPTION_WENT_PAST_DUE . '_subscription_controller', array ( 
				$this, 
				'subscription_past_due' 
		), 10, 3 );
	
	}

	public static function init()
	{
		add_filter( 'bfwc_register_route_controllers', __CLASS__ . '::register_controller' );
	}

	public function get_notification_kinds()
	{
		return apply_filters( __CLASS__ . '_notification_kinds', array ( 
				WebhookNotification::SUBSCRIPTION_CANCELED => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_CHARGED_SUCCESSFULLY => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_CHARGED_UNSUCCESSFULLY => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_EXPIRED => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_TRIAL_ENDED => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_WENT_ACTIVE => array ( 
						$this, 
						'process_subscription_webhook' 
				), 
				WebhookNotification::SUBSCRIPTION_WENT_PAST_DUE => array ( 
						$this, 
						'process_subscription_webhook' 
				) 
		) );
	}

	/**
	 * Validate the webhook notification.
	 *
	 * @param Braintree_WebhookNotification $notification 
	 */
	public function validate_webhook( $notification )
	{
		switch ($notification->kind) {
			case WebhookNotification::SUBSCRIPTION_CANCELED :
			case WebhookNotification::SUBSCRIPTION_CHARGED_SUCCESSFULLY :
			case WebhookNotification::SUBSCRIPTION_CHARGED_UNSUCCESSFULLY :
			case WebhookNotification::SUBSCRIPTION_EXPIRED :
			case WebhookNotification::SUBSCRIPTION_TRIAL_ENDED :
			case WebhookNotification::SUBSCRIPTION_WENT_ACTIVE :
			case WebhookNotification::SUBSCRIPTION_WENT_PAST_DUE :
				global $bfwc_webhook_subscription_post;
				$id = absint( $notification->subscription->id );
				if ( ! $post = get_post( $id ) ) {
					throw new Exception( sprintf( __( 'Subscription %s was not found in your Wordpress database. 
							Subscription cancellation webhook could not be processed.', 'braintree' ), $id ), 404 );
				}
				$bfwc_webhook_subscription_post = $post;
				break;
		}
	}

	/**
	 *
	 * @param array $controllers 
	 */
	public static function register_controller( $controllers )
	{
		// only register this controller if WooCommerce Subscriptions is active.
		if ( bt_manager()->is_woocommerce_subscriptions_active() ) {
			$controllers[] = __CLASS__;
		}
		return $controllers;
	}

	/**
	 *
	 * @param WebhookNotification $notification 
	 */
	public function process_subscription_webhook( $notification )
	{
		global $bfwc_api_message;
		
		$wc_subscription = wcs_get_subscription( $notification->subscription->id );
		
		do_action( "bfwc_{$notification->kind}_subscription_controller", $wc_subscription, $notification->subscription, $notification );
	}

	/**
	 *
	 * @param WC_Subscription $wc_subscription 
	 * @param Braintree_Subscription $subscription 
	 * @param Braintree_WebhookNotification $notification 
	 */
	public function subscription_cancelled( $wc_subscription, $subscription, $notification )
	{
		global $bfwc_api_message;
		
		if ( ! $wc_subscription->has_status( 'cancelled' ) ) {
			$wc_subscription->update_status( 'cancelled' );
			$bfwc_api_message = sprintf( __( 'Subscription %s has been cancelled.', 'braintree' ), $wc_subscription->get_order_number() );
		} else {
			$bfwc_api_message = sprintf( __( 'Subscription %s has already been cancelled.', 'braintree' ), $wc_subscription->get_order_number() );
		}
	}

	/**
	 *
	 * @param WC_Subscription $wc_subscription 
	 * @param Braintree_Subscription $subscription 
	 * @param WebhookNotification $notification 
	 */
	public function charged_successfully( $wc_subscription, $subscription, $notification )
	{
		global $wpdb, $bfwc_api_message;
		$id = $subscription->id;
		
		$order = $wc_subscription->order;
		$num_of_trans = count( $subscription->transactions );
		$transaction = $num_of_trans > 0 ? $subscription->transactions[ 0 ] : null;
		
		if ( ! $transaction ) {
			$bfwc_api_message = sprintf( __( 'Subscription %s does not have any transactions associated with it yet. The 
					charged successfully webhook should only be called once a recurring payment has been processed by Braintree.', 'braintree' ), $subscription->id );
			return;
		}
		
		/* If there is only one transaction, and the subscription does not have a trial and is not synced and doesn't have a transaction Id then don't create renewal order. */
		$create_order = $num_of_trans === 1 && ! bwcs_subscription_has_trial( $wc_subscription ) && ! bwcs_subscription_is_synched( $wc_subscription ) && ! $order->get_transaction_id() ? false : true;
		
		if ( ! $create_order ) {
			update_post_meta( $wc_subscription->id, '_transaction_id', $transaction->id );
			
			// Original order doesn't have a transaction so add it here.
			$order->payment_complete( $transaction->id );
			
			if ( ! $wc_subscription->has_status( 'active' ) ) {
				$wc_subscription->update_status( 'active' );
			}
			
			$wc_subscription->update_dates( array ( 
					'next_payment' => $wc_subscription->calculate_date( 'next_payment' ) 
			) );
			
			$bfwc_api_message = sprintf( __( 'Recurring payment for subscription %s charged by Braintree on %s.', 'braintree' ), $subscription->id, $transaction->createdAt->format( 'Y-m-d H:i:s' ) );
		} else {
			
			/*
			 * Look for an existing order to make sure duplicates aren't created, resulting in incorrect financial reporting.
			 * Another cause for a duplicate could be during webhook testing.
			 */
			$results = $wpdb->get_row( $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key = '_transaction_id' AND meta_value = %s", $transaction->id ) );
			
			if ( $results ) {
				$renewal_order = wc_get_order( $results->post_id );
				$bfwc_api_message = sprintf( __( 'Renewal order %s already created for subscription %s.', 'braintree' ), $renewal_order->get_order_number(), $wc_subscription->get_order_number() );
				return;
			}
			// create a renewal order.
			$renewal_order = wcs_create_renewal_order( $wc_subscription );
			$renewal_order->add_order_note( sprintf( __( 'Recurring payment charged for subscription by Braintree on %s.', 'braintree' ), $transaction->createdAt->format( 'Y-m-d H:i:s' ) ) );
			
			// save the payment method data for the renewal order.
			update_post_meta( $renewal_order->id, '_payment_method', $wc_subscription->payment_method );
			update_post_meta( $renewal_order->id, '_payment_method_title', $wc_subscription->payment_method_title );
			update_post_meta( $renewal_order->id, '_payment_method_token', $wc_subscription->payment_method_token );
			
			$renewal_order->payment_complete( $transaction->id );
			
			if ( ! $wc_subscription->has_status( 'active' ) ) {
				$wc_subscription->update_status( 'active' );
			}
			
			// subscription started immediately and the order already has a transaction Id so subtract renewal order from original order.
			if ( $create_order && $num_of_trans === 1 ) {
				$order->set_total( $order->get_total() - $wc_subscription->get_total() );
			}
			
			$wc_subscription->update_dates( array ( 
					'next_payment' => $wc_subscription->calculate_date( 'next_payment' ) 
			) );
			
			$bfwc_api_message = sprintf( __( 'Renewal order %s created for subscription %s.', 'braintree' ), $renewal_order->get_order_number(), $wc_subscription->get_order_number() );
		}
	}

	/**
	 * Update the subscription status for a failed renewal payment.
	 * The WC_Subscription status is changed to on-hold.
	 *
	 * @param WC_Subscription $wc_subscription 
	 * @param Braintree_Subscription $subscription 
	 * @param Braintree_WebhookNotification $notification 
	 */
	public function charged_unsuccessfully( $wc_subscription, $subscription, $notification )
	{
		global $bfwc_api_message;
		$bfwc_api_message = '';
		
		// payment for subscription failed so put it on hold.
		try {
			$wc_subscription->update_status( apply_filters( 'bfwc_webhook_failed_payment_status', 'on-hold', $wc_subscription ) );
			$wc_subscription->add_order_note( sprintf( __( 'Status for subscription changed to on-hold because of a failed recurring payment.', 'braintree' ) ) );
			$bfwc_api_message = sprintf( __( 'Subscription %s placed on-hold due to a failed recurring payment.', 'braintree' ), $wc_subscription->get_order_number() );
		} catch ( Exception $e ) {
			throw new Exception( $e->getMessage(), 200 );
		}
	}

	/**
	 *
	 * @param WC_Subscription $wc_subscription 
	 * @param Braintree_Subscription $subscription 
	 * @param Braintree_WebhookNotification $notification 
	 */
	public function subscription_past_due( $wc_subscription, $subscription, $notification )
	{
		global $bfwc_api_message;
		$wc_subscription = wcs_get_subscription( $subscription->id );
		
		$wc_subscription->update_status( 'wc-on-hold' );
		
		$bfwc_api_message = sprintf( __( 'Subscription %s is past due and has been set to on hold', 'braintree' ), $wc_subscription->get_order_number() );
	}

	/**
	 *
	 * @param WC_Subscription $wc_subscription 
	 * @param Braintree_Subscription $subscription 
	 * @param Braintree_WebhookNotification $notification 
	 */
	public function subscription_expired( $wc_subscription, $subscription, $notification )
	{
		global $bfwc_api_message;
		$wc_subscription = wcs_get_subscription( $subscription->id );
		
		$wc_subscription->update_status( 'wc-expired' );
		
		$bfwc_api_message = sprintf( __( 'Subscription %s is expired.', 'braintree' ), $wc_subscription->get_order_number() );
	}

}
Braintree_Gateway_WCS_Controller::init();