<?php
class Braintree_Gateway_Frontend_Scripts
{
	public static $scripts = array ();
	public static $styles = array ();
	public static $prefix = 'braintree-';

	public static function init()
	{
		if ( is_admin() ) {
			return;
		}
		
		define( 'BRAINTREE_JS_DROPIN', 'https://js.braintreegateway.com/js/braintree-2.30.0.min.js' );
		define( 'BRAINTREE_JS_V3_3DS', 'https://js.braintreegateway.com/web/3.6.3/js/three-d-secure.min.js' );
		define( 'BRAINTREE_JS_V3_CLIENT', 'https://js.braintreegateway.com/web/3.6.3/js/client.min.js' );
		define( 'BRAINTREE_JS_V3_HOSTED', 'https://js.braintreegateway.com/web/3.6.3/js/hosted-fields.min.js' );
		define( 'BRAINTREE_PAYPAL_JS', 'https://js.braintreegateway.com/web/3.6.3/js/paypal.min.js' );
		define( 'BRAINTREE_V3_DATA_COLLECTOR', 'https://js.braintreegateway.com/web/3.6.3/js/data-collector.min.js' );
		define( 'BRAINTREE_V3_UNIONPAY', 'https://js.braintreegateway.com/web/3.6.3/js/unionpay.min.js' );
		define( 'BRAINTREE_V3_APPLEPAY', 'https://js.braintreegateway.com/web/3.6.3/js/apple-pay.min.js' );
		
		add_action( 'wp_enqueue_scripts', __CLASS__ . '::load_scripts', 999 );
		
		add_action( 'wp_print_scripts', __CLASS__ . '::localize_scripts' );
		
		add_action( 'wp_print_footer_scripts', __CLASS__ . '::localize_scripts' );
		
		add_action( 'braintree_before_localize_frontend_scripts', __CLASS__ . '::initialize_cart_totals' );
	}

	public static function load_scripts()
	{
		// ensure WooCommerce has been activated.
		if ( ! did_action( 'before_woocommerce_init' ) ) {
			return;
		}
		
		$js_path = bt_manager()->plugin_assets_path() . 'js/frontend/';
		$css_path = bt_manager()->plugin_assets_path() . 'css/';
		$version = bt_manager()->version;
		
		self::register_script( self::$prefix . 'external-dropin', BRAINTREE_JS_DROPIN, array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'client', BRAINTREE_JS_V3_CLIENT, array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'hosted', BRAINTREE_JS_V3_HOSTED, array (
				'jquery' 
		), $version, true );
		self::register_script( self::$prefix . '3ds-js', BRAINTREE_JS_V3_3DS, array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'paypal-external', BRAINTREE_PAYPAL_JS, array (
				self::$prefix . 'client' 
		), $version, true );
		
		self::register_script( self::$prefix . 'data-collector', BRAINTREE_V3_DATA_COLLECTOR, array (
				self::$prefix . 'client' 
		), $version, true );
		
		self::register_script( self::$prefix . 'union-pay', BRAINTREE_V3_UNIONPAY, array (
				self::$prefix . 'client' 
		), $version, true );
		
		self::register_script( self::$prefix . 'applepay-external', BRAINTREE_V3_APPLEPAY, array (
				self::$prefix . 'client' 
		), $version, true );
		
		self::register_script( self::$prefix . 'change-payment-method', $js_path . 'change-payment-method.js', array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'payment-methods', $js_path . 'braintree-payment-methods.js', array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'message-handler', $js_path . 'message-handler.js', array (
				'jquery' 
		), $version, true );
		
		self::register_script( self::$prefix . 'form-handler', $js_path . 'form-handler.js', array (
				'jquery' 
		), $version, true );
		
		$form = bwc_get_custom_form();
		self::register_script( self::$prefix . 'custom-js', $form [ 'javascript' ], array (
				'jquery', 
				self::$prefix . 'hosted-fields' 
		), $version, true );
		self::register_style( self::$prefix . 'custom-css', $form [ 'css' ], array (), $version, null );
		
		if ( ! empty( $form [ 'external_css' ] ) ) {
			self::register_style( self::$prefix . 'custom-external-css', $form [ 'external_css' ], array (), $version, null );
		}
		
		$button = bwc_get_paypal_button();
		
		self::register_style( self::$prefix . 'paypal-css', $button [ 'css' ], array (), $version );
		
		self::register_style( self::$prefix . 'styles', $css_path . 'braintree.css', array (), $version );
		
		self::register_script( self::$prefix . 'hosted-fields', $js_path . 'hosted-fields.js', array (
				self::$prefix . 'client', 
				self::$prefix . 'hosted' 
		), $version, true );
		
		self::register_script( self::$prefix . 'dropin', $js_path . 'dropin.js', array (
				self::$prefix . 'external-dropin' 
		), $version, true );
		
		self::register_script( self::$prefix . 'paypal', $js_path . 'paypal.js', array (
				self::$prefix . 'paypal-external' 
		), $version, true );
		
		self::register_script( self::$prefix . 'applepay', $js_path . 'applepay.js', array (
				self::$prefix . 'applepay-external' 
		), $version, true );
		
		self::enqueue_scripts_for_page();
	}

	public static function enqueue_scripts_for_page()
	{
		if ( is_checkout() || is_add_payment_method_page() || bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			wp_enqueue_style( self::$prefix . 'styles' );
			wp_enqueue_style( self::$prefix . 'paypal-css' );
			wp_enqueue_script( self::$prefix . 'payment-methods' );
			wp_enqueue_script( self::$prefix . 'message-handler' );
			wp_enqueue_script( self::$prefix . 'form-handler' );
			if ( bwc_is_custom_form() ) {
				wp_enqueue_script( self::$prefix . 'hosted-fields' );
				if ( bwc_is_3ds_active() ) {
					wp_enqueue_script( self::$prefix . '3ds-js' );
				}
				if ( bwc_is_advanced_fraud_tools() ) {
					wp_enqueue_script( self::$prefix . 'data-collector' );
				}
				wp_enqueue_script( self::$prefix . 'custom-js' );
				wp_enqueue_style( self::$prefix . 'custom-css' );
				wp_enqueue_style( self::$prefix . 'custom-external-css' );
			} else {
				wp_enqueue_script( self::$prefix . 'dropin' );
			}
			if ( bwc_is_paypal_only() || bwc_is_paypal_enabled() ) {
				
				wp_enqueue_script( self::$prefix . 'paypal' );
			}
			if ( bwc_is_applepay_enabled() ) {
				wp_enqueue_script( self::$prefix . 'applepay' );
			}
		}
		if ( bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			wp_enqueue_script( self::$prefix . 'change-payment-method' );
		}
	}

	/**
	 * Register and enqueue a script.
	 *
	 * @param string $handle        	
	 * @param string $src        	
	 * @param array $deps        	
	 * @param string $version        	
	 * @param bool $in_footer        	
	 */
	public static function enqueue_script( $handle, $src = '', $deps = array(), $version = '', $in_footer = false )
	{
		if ( ! in_array( $handle, self::$scripts ) ) {
			self::register_script( $handle, $src, $deps, $version, $in_footer );
		}
		wp_enqueue_script( $handle, $src, $deps, $version, $in_footer );
	}

	/**
	 * Register a script for enqueue later.
	 *
	 * @param string $handle        	
	 * @param string $src        	
	 * @param array $deps        	
	 * @param string $version        	
	 * @param bool $in_footer        	
	 */
	public static function register_script( $handle, $src, $deps = array(), $version, $in_footer )
	{
		self::$scripts [] = $handle;
		
		wp_register_script( $handle, $src, $deps, $version, $in_footer );
	}

	/**
	 * Register a style sheet for enquement later.
	 *
	 * @param string $handle        	
	 * @param string $src        	
	 * @param array $deps        	
	 * @param string $version        	
	 * @param string $media        	
	 */
	public static function register_style( $handle, $src, $deps = array(), $version, $media = null )
	{
		if ( ! in_array( $handle, self::$styles ) ) {
			self::$styles [] = $handle;
		}
		
		wp_register_style( $handle, $src, $deps, $version, $media );
	}

	/**
	 * Enqueue a style sheet.
	 *
	 * @param string $handle        	
	 * @param string $src        	
	 * @param array $deps        	
	 * @param string $version        	
	 * @param string $media        	
	 */
	public static function enqueue_style( $handle, $src, $deps = array(), $version = '', $media = null )
	{
		if ( ! in_array( $handle, self::$styles ) ) {
			self::register_style( $handle, $src, $deps, $version, $media = null );
		}
		
		wp_enqueue_style( $handle, $src, $deps, $version, $media );
	}

	public static function localize_scripts()
	{
		// ensure WooCommerce has been activated.
		if ( ! did_action( 'before_woocommerce_init' ) ) {
			return;
		}
		
		do_action( 'braintree_before_localize_frontend_scripts' );
		
		foreach ( self::$scripts as $handle ) {
			if ( wp_script_is( $handle, 'enqueued' ) && ( $data = self::get_script_data( $handle ) ) ) {
				
				$object_name = str_replace( '-', '_', $handle ) . '_vars';
				
				wp_localize_script( $handle, $object_name, $data );
			}
		}
	}

	public static function get_script_data( $handle )
	{
		switch( $handle) {
			case self::$prefix . 'dropin' :
			case self::$prefix . 'hosted-fields' :
			case self::$prefix . 'paypal' :
			case self::$prefix . 'applepay' :
				global $bfwc_frontend_client_token;
				
				if ( ! $bfwc_frontend_client_token ) {
					$bfwc_frontend_client_token = bt_manager()->get_client_token( array (
							'merchantAccountId' => bwc_get_merchant_account() 
					) );
				}
				
				$data = array (
						'client_token' => $bfwc_frontend_client_token, 
						'order_total' => WC()->cart->total, 
						'_3ds' => array (
								'enabled' => bwc_is_3ds_active() 
						), 
						'advanced_fraud' => array (
								'enabled' => bwc_is_advanced_fraud_tools() 
						), 
						'environment' => bt_manager()->get_environment(), 
						'page' => array (
								'checkout_page' => bwc_is_checkout(), 
								'change_payment_page' => bwcs_is_change_payment_method() 
						) 
				);
				
				if ( $handle === self::$prefix . 'hosted-fields' ) {
					$data [ 'gateway_id' ] = bwc_get_gateway_id( 'braintree' );
					$data [ 'custom_fields' ] = bwc_get_custom_form_fields();
					$data [ 'dynamic_card_display' ] = array (
							'enabled' => bwc_is_dynamic_card_display() 
					);
					$data [ 'form_styles' ] = json_decode( bt_manager()->get_option( 'custom_form_styles' ), true );
					$data [ 'loader' ] = array (
							'enabled' => bt_manager()->is_active( 'enable_loader' ), 
							'css' => json_decode( bt_manager()->get_option( 'custom_form_loader_css' ), true ), 
							'message' => bt_manager()->get_option( 'custom_form_loader_message' ) 
					);
					if ( bwc_is_3ds_active() ) {
						$data [ '_3ds' ] [ 'modal_html' ] = bwc_get_3ds_modal_html();
					}
				}
				if ( $handle === self::$prefix . 'dropin' ) {
					$data [ 'gateway_id' ] = bwc_get_gateway_id( 'braintree' );
				}
				if ( $handle === self::$prefix . 'paypal' ) {
					
					$data [ 'gateway_id' ] = bwc_get_gateway_id( 'paypal' );
					$data [ 'html' ] = bwc_get_paypal_html();
					$data [ 'form' ] [ 'submit' ] = bt_manager()->is_active( 'paypal_submit_form' );
					if ( bwc_paypal_checkout_flow() && is_checkout() ) {
						$data [ 'options' ] = array (
								'amount' => WC()->cart->total, 
								'flow' => 'checkout', 
								'currency' => get_woocommerce_currency(), 
								'displayName' => bt_manager()->get_option( 'paypal_display_name' ) 
						);
					} else {
						$data [ 'options' ] = array (
								'flow' => 'vault', 
								'displayName' => bt_manager()->get_option( 'paypal_display_name' ) 
						);
					}
				}
				if ( $handle === self::$prefix . 'applepay' ) {
					$data [ 'gateway_id' ] = bwc_get_gateway_id( 'applepay' );
					$data [ 'store_name' ] = bt_manager()->get_option( 'applepay_store_name' );
					if ( ! $data [ 'order_total' ] ) {
						$data [ 'order_total' ] = 0.01; // apple pay requires at least one cent.
					}
				}
				break;
			case self::$prefix . 'change-payment-method' :
				$data [ 'loader' ] = array (
						'enabled' => bt_manager()->is_active( 'enable_loader' ), 
						'css' => json_decode( bt_manager()->get_option( 'custom_form_loader_css' ), true ), 
						'message' => bt_manager()->get_option( 'custom_form_loader_message' ) 
				);
				break;
			case self::$prefix . 'message-handler' :
				$data [ 'messages' ] = bfwc_get_combined_error_messages();
				break;
			case self::$prefix . 'payment-methods' :
				$data [ 'style' ] = bwc_saved_payment_method_style();
				break;
			default :
				$data = false;
		}
		return $data;
	}

	public static function initialize_cart_totals()
	{
		if ( is_checkout() ) {
			WC()->cart->calculate_totals(); // Prevent WC()->cart->total from
				                                // returning 0.
		}
	}
}
Braintree_Gateway_Frontend_Scripts::init();