<?php
class WC_Product_Braintree_Variable_Subscription extends WC_Product_Variable
{

	public function __construct( $product )
	{
		parent::__construct( $product );
		
		$this->subscription_period = 'month';
		$this->product_type = 'braintree-variable-subscription';
		
		add_action( 'woocommerce_add_to_cart_handler', array ( 
				$this, 
				'add_to_cart_handler' 
		), 10, 2 );
	}

	public function get_price_html( $price = '' )
	{
		
		if ( $this->min_subscription_price_variation_id !== $this->max_subscription_price_variation_id ) {
			$low_variation = $this->get_child( $this->min_subscription_price_variation_id );
			$high_variation = $this->get_child( $this->max_subscription_price_variation_id );
			
			$text = sprintf( __( 'From %s to %s', 'braintree' ), $low_variation->get_price_html(), $high_variation->get_price_html() );
		
		} else {
			$variation = $this->get_child( $this->min_subscription_price_variation_id );
			$text = $variation->get_price_html();
		}
		return $text;
	
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Product_Variable::get_child()
	 * @return WC_Product_Braintree_Subscription_Variation
	 */
	public function get_child( $child_id )
	{
		return wc_get_product( $child_id, array ( 
				'product_type' => 'braintree_subscription_variation', 
				'parent_id' => $this->id, 
				'parent' => $this 
		) );
	}

	public function get_variation_subscription_prices()
	{
		$min_price = $this->min_subscription_price;
		$max_price = $this->max_subscription_price;
		
		// sync if min or max is not saved.
		if ( ! $min_price || ! $max_price ) {
			self::sync( $this->id );
		}
		
		return array ( 
				'min' => $min_price, 
				'max' => $max_price 
		);
	}

	public function get_plans()
	{
		$var = 'braintree_' . bt_manager()->get_environment() . '_plans';
		return $this->$var;
	}

	/**
	 * Return the subscription signup fee.
	 *
	 * @return string $fee;
	 */
	public function get_signup_fee()
	{
		return $this->subscription_sign_up_fee ? $this->subscription_sign_up_fee : 0;
	}

	/**
	 * Return true if shipping is only charged once for the product.
	 * If there is a trial period then shipping must be included
	 * in all charges if there is any shipping.
	 */
	public function is_one_time_shipping()
	{
		return $this->needs_shipping() && $this->subscription_one_time_shipping === 'yes' && ! $this->subscription_trial_length;
	}

	public static function sync( $product_id )
	{
		$children = get_posts( array ( 
				'post_parent' => $product_id, 
				'posts_per_page' => - 1, 
				'post_type' => 'product_variation', 
				'post_status' => 'publish' 
		) );
		
		if ( ! $children ) {
			return;
		}
		
		$price_types = array ( 
				'subscription_price' 
		);
		
		foreach ( $price_types as $price_type ) {
			foreach ( $children as $child ) {
				
				$child_price = get_post_meta( $child->ID, '_' . $price_type, true );
				
				// if the min price_type is null or it's greater than the child_pricetype, change it's value to the child_price.
				if ( is_null( ${"min_$price_type"} ) || ${"min_$price_type"} > $child_price ) {
					${"min_$price_type"} = $child_price;
					${"min_{$price_type}_variation_id"} = $child->ID;
				}
				
				if ( is_null( ${"max_$price_type"} ) || ${"max_$price_type"} < $child_price ) {
					${"max_$price_type"} = $child_price;
					${"max_{$price_type}_variation_id"} = $child->ID;
				}
			
			}
			
			update_post_meta( $product_id, "_min_$price_type", ${"min_$price_type"} );
			update_post_meta( $product_id, "_max_$price_type", ${"max_$price_type"} );
			
			update_post_meta( $product_id, "_min_{$price_type}_variation_id", ${"min_{$price_type}_variation_id"} );
			update_post_meta( $product_id, "_max_{$price_type}_variation_id", ${"max_{$price_type}_variation_id"} );
		}
	}

	public function add_to_cart_handler( $product_type, $product )
	{
		if ( $this->product_type === $product_type ) {
			$product_type = 'variable';
		}
		return $product_type;
	}
}