<?php
/**
 * 
 * @author Payment Plugins
 * @copyright 2016 Payment Plugins
 *
 *@property string $braintree_plan Braintree plan Id.
 *@property string $merchant_account_id merchant account Id to be used on the subscription.
 */
class Braintree_Gateway_WC_Subscription extends WC_Order
{
	
	/**
	 *
	 * @var WC_Order
	 */
	public $order;

	public function __construct( $order )
	{
		parent::__construct( $order );
		
		$this->order = $this->post->post_parent ? wc_get_order( $this->post->post_parent ) : null;
	}

	/**
	 * Update the Subscription status.
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Abstract_Order::update_status()
	 */
	public function update_status( $new_status, $note = '', $manual = false )
	{
		global $wpdb;
		
		$new_status = ( strpos( $new_status, 'wc-' ) !== false ) ? substr( $new_status, 3 ) : $new_status;
		$old_status = $this->get_status();
		$new_status_key = 'wc-' . $new_status;
		$old_status_key = 'wc-' . $old_status;
		
		try {
			
			// only update if the status is changed
			if ( $old_status !== $new_status ) {
				switch( $new_status) {
					case 'active' :
						if ( $this->has_status( array (
								'cancelled', 
								'expired', 
								'past-due' 
						) ) ) {
							throw new Exception( sprintf( __( 'Subscriptions with status %s cannot be changed to Active', 'braintree' ), bfwc_get_subscription_status_name( $this->post_status ) ) );
						}
						break;
					case 'cancelled' :
						if ( $this->has_status( array (
								'expired' 
						) ) ) {
							throw new Exception( sprintf( __( 'Subscriptions that are expired cannot be changed to %s.', 'braintree' ), bfwc_get_subscription_status_name( $this->post_status ) ) );
						}
						break;
				}
				
				// update the subscription status.
				$wpdb->update( $wpdb->posts, array (
						'post_status' => $new_status_key 
				), array (
						'ID' => $this->id 
				) );
				
				// allow functionality to hook into the status change trigger.
				do_action( 'bfwc_subscription_status_' . $new_status, $old_status_key, $this );
				
				// made it this far so status was updated.
				$this->add_order_note( sprintf( __( 'Subscription status changed from %1$s to %2$s.', 'braintree' ), bfwc_get_subscription_status_name( $old_status ), bfwc_get_subscription_status_name( $new_status ) ) );
			
			}
		} catch( Exception $e ) {
			// update the post status to what it was before since there was an exception thrown.
			$wpdb->update( $wpdb->posts, array (
					'post_status' => $old_status_key 
			), array (
					'ID' => $this->id 
			) );
			
			$this->add_order_note( sprintf( __( 'Unable to change subscription status from %1$s to %2$s. Message: %3$s', 'braintee' ), bfwc_get_subscription_status_name( $old_status ), bfwc_get_subscription_status_name( $new_status ), $e->getMessage() ) );
		
		}
	}

	public function payment_complete( $transaction_id = '' )
	{
		if ( ! empty( $transaction_id ) ) {
			update_post_meta( $this->id, '_transaction_id', $transaction_id );
		}
		
		$this->update_status( 'active' );
	}

	/**
	 * Does the subscription have a length or does it never expire.
	 *
	 * @return bool
	 */
	public function never_expires()
	{
		return 0 === ( int ) $this->subscription_length;
	}

	/**
	 * Return true if the subscription has a trial period.
	 *
	 * @return bool
	 */
	public function has_trial()
	{
		return 0 !== ( int ) $this->subscription_trial_length;
	}

	public function get_trial_period()
	{
		return $this->subscription_trial_period;
	}

	public function get_trial_length()
	{
		return $this->subscription_trial_length;
	}

	public function get_date_key( $type )
	{
		return strpos( '_date', $type ) !== false ? $type : "{$type}_date";
	}

	/**
	 * Return the datetime object for the specified date.
	 * All dates are returned in UTC.
	 *
	 * @return DateTime
	 */
	public function get_date( $type )
	{
		$type = $this->get_date_key( $type );
		
		// all dates are stored as UTC in the database.
		$date = DateTime::createFromFormat( 'Y-m-d H:i:s', $this->$type, new DateTimeZone( 'UTC' ) );
		return $date;
	}

	/**
	 *
	 * @param string $type        	
	 * @param
	 *        	mixed string|DateTime $date
	 */
	public function update_date( $type, $date )
	{
		if ( $date instanceof DateTime ) {
			$date_string = $date->format( 'Y-m-d H:i:s' );
		} else {
			$date_string = $date;
		}
		$type = '_' . $this->get_date_key( $type );
		update_post_meta( $this->id, $type, $date_string );
	}

	public function calculate_date( $type )
	{
		switch( $type) {
			case 'next_payment' :
				$last_payment = $this->get_last_pay;
				
				break;
		}
	}

	/**
	 * Calculate the next payment date given a date.
	 *
	 * @param DateTime $date        	
	 */
	public function get_next_payment_date( $date )
	{
		switch( $this->subscription_period) {
			case 'day' :
				$date->add( new DateInterval( 'P' . $this->subscription_period_interval . 'D' ) );
				break;
			case 'month' :
				$date->add( new DateInterval( 'P' . $this->subscription_period_interval . 'M' ) );
				break;
		}
		return $date;
	}

	/**
	 * Return the descriptor for the type provided.
	 * Valid types are <strong>name</strong>, <strong>phone</strong>, <strong>url</strong>
	 *
	 * @param unknown $type        	
	 */
	public function get_descriptor( $type )
	{
		$descriptors = $this->descriptor;
		if ( $descriptors ) {
			return isset( $descriptors [ $type ] ) ? $descriptors [ $type ] : '';
		}
		return '';
	}

	/**
	 * Return true if descriptors have been configured for the subscription.
	 */
	public function has_descriptors()
	{
		return ( bool ) $this->descriptors;
	}

	public function update_payment_method_title( $title )
	{
		update_post_meta( $this->id, '_payment_method_title', $title );
	}

	public function update_payment_method_token( $token = '' )
	{
		update_post_meta( $this->id, '_payment_method_token', $token );
	}

	/**
	 *
	 * @param bool $bool        	
	 */
	public function set_created( $bool )
	{
		update_post_meta( $this->id, '_created_in_braintree', $bool );
	}

	/**
	 * Return true if the subscription has been created within Braintree.
	 */
	public function is_created()
	{
		return $this->created_in_braintree;
	}

	public function get_formatted_total()
	{
		$total = $this->get_total();
		
		$total_string = sprintf( '%s %s %s', get_woocommerce_currency_symbol( $this->order_currency ), $total, bfwcs_get_interval_string( $this->subscription_period_interval, $this->subscription_period ) );
		return apply_filters( 'bfwc_subscription_formatted_order_total', $total_string, $this );
	}

	/**
	 * Return a formatted date string using the timezone that the subscription was created in.
	 *
	 * @param string $type        	
	 * @param string $format        	
	 */
	public function get_formatted_date( $type, $format = null )
	{
		$format = $format ? $format : get_option( 'date_format' );
		$date = $this->get_date( $type );
		
		switch( $type) {
			case 'next_payment' :
				if ( ! $this->has_child_orders() ) {
					$date = $this->get_next_payment_date( $date );
				}
				break;
		}
		if ( $date ) {
			$date->setTimezone( new DateTimeZone( $this->subscription_time_zone ? $this->subscription_time_zone : 'UTC' ) );
			return $date->format( $format );
		}
		switch( $type) {
			case 'end' :
				return __( 'Never Expires', 'braintree' );
		}
	}

	/**
	 * return true if child orders have been processed for the subscription.
	 */
	public function has_child_orders()
	{
		global $wpdb;
		
		$result = $wpdb->get_results( $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key = '_subscription_id' AND meta_value = %s", $this->id ) );
		return ! empty( $result );
	}

	public function get_payment_method_to_display()
	{
		return apply_filters( 'bfwc_payment_method_to_display', $this->payment_method_title, $this );
	}

	public function get_view_subscription_url()
	{
		$url = wc_get_endpoint_url( 'view-subscription', $this->id, wc_get_page_permalink( 'myaccount' ) );
		return apply_filters( 'bfwc_subscription_view_url', $url, $this );
	}
}