<?php
if ( ! class_exists( 'WC_Braintree_Payment_Gateway' ) ) {
	return;
}
class WC_PayPal_Payment_Gateway extends WC_Braintree_Payment_Gateway
{
	const ID = 'braintree_paypal_payments';
	
	/**
	 * Nonce id contained in the $_POST.
	 *
	 * @var string
	 */
	public static $nonce_id = 'braintree_paypal_payments_nonce';
	
	/**
	 * Device data field key located in $_POST.
	 *
	 * @var string
	 */
	public static $device_data_id = 'braintree_paypal_device_data';
	
	/**
	 * The key used in $_POST to identify the selected payment method token.
	 *
	 * @var string
	 */
	public static $token_id = 'braintree_paypal_payments_method_token';

	public function __construct()
	{
		if ( bwc_is_paypal_only() ) {
			// if PayPal only, make sure PayPal is enabled, regardless of
			// 'enable_paypal' setting.
			$this->enabled = 'yes';
		} else {
			$this->enabled = bwc_is_paypal_enabled() ? 'yes' : 'no';
		}
		$this->title = bt_manager()->get_option( 'paypal_gateway_title' );
		$this->id = self::ID;
		$this->method_title = __( 'PayPal Braintree Payment Gateway', 'braintree' );
		$this->has_fields = true;
		$this->actions();
		$this->set_supports();
	}

	public static function init()
	{
		add_filter( 'bwc_add_payment_gateways', __CLASS__ . '::add_braintree_gateway' );
		
		add_filter( 'woocommerce_payment_gateways', __CLASS__ . '::add_gateway' );
		
		add_action( 'wp_loaded', __CLASS__ . '::maybe_delete_payment_method' );
		
		add_filter( 'woocommerce_payment_complete_order_status', __CLASS__ . '::maybe_update_order_status', 99, 2 );
		
		add_action( 'woocommerce_subscription_failing_payment_method_updated_' . static::ID, __CLASS__ . '::update_failing_payment_method', 10, 2 );
		
		add_action( 'woocommerce_after_checkout_validation_' . static::ID, __CLASS__ . '::maybe_save_payment_method' );
		
		add_action( 'bfwc_before_process_order_' . static::ID, __CLASS__ . '::maybe_save_payment_method' );
	}

	public static function add_braintree_gateway( $gateways )
	{
		$gateways [] = __CLASS__;
		return $gateways;
	}

	public static function add_gateway( $gateways )
	{
		$gateways [] = __CLASS__;
		return $gateways;
	}

	public function payment_fields()
	{
		if ( bwc_is_checkout() || bwcs_is_change_payment_method() || bfwcs_is_change_payment_method() || bfwcs_is_pay_for_subscription_request() ) {
			
			bwc_get_template( 'checkout/paypal-checkout.php', array (
					'gateway' => $this 
			) );
		} elseif ( is_add_payment_method_page() ) {
			
			bwc_get_template( 'paypal.php', array (
					'has_methods' => false, 
					'gateway' => $this 
			) );
		}
	}

	public function woocommerce_gateway_icon( $icon, $id )
	{
		if ( $id === $this->id ) {
			$icon = '<span class="paypal-icon paypal"></span>';
		}
		return $icon;
	}

	public function saved_payment_method_list( $saved_methods, $user_id )
	{
		$methods = bwc_get_user_paypal_payment_methods( $user_id );
		
		return bwc_saved_payment_methods_list( $saved_methods, $methods, $this->id );
	}
}
WC_PayPal_Payment_Gateway::init();