<?php
use Braintree\Exception;

/**
 *
 * @author Payment Plugins
 * @copyright Payment Plugins 2016
 */
class Braintree_Gateway_Donation_Shortcode
{
	
	private $scripts = array (
			'js' => array (), 
			'css' => array () 
	);
	
	private $_cookie;
	
	private $donation = '';
	
	public $is_transaction = false;
	
	private $ajax_action = 'process_braintree_donation';
	
	private $ajax_recurring_action = 'process_braintree_recurring_donation';
	
	private $_transient = '';
	
	private $messages = array ();
	
	private $is_modal_active;

	public function __construct()
	{
		$this->_cookie = 'braintree_donation_' . COOKIEHASH;
		
		add_shortcode( 'braintree_donations', array (
				$this, 
				'output_transaction_page' 
		) );
		add_shortcode( 'braintree_recurring_donation', array (
				$this, 
				'output_recurring_page' 
		) );
		add_action( "wp_ajax_{$this->ajax_action}", array (
				$this, 
				'process_donation' 
		) );
		add_action( "wp_ajax_nopriv_{$this->ajax_action}", array (
				$this, 
				'process_donation' 
		) );
		add_action( "wp_ajax_{$this->ajax_recurring_action}", array (
				$this, 
				'process_recurring_donation' 
		) );
		add_action( "wp_ajax_nopriv_{$this->ajax_recurring_action}", array (
				$this, 
				'process_recurring_donation' 
		) );
		add_action( 'wp', array (
				$this, 
				'init_cookie' 
		) );
		add_action( 'wp_enqueue_scripts', array (
				$this, 
				'register_scripts' 
		) );
	}

	public function output( $args )
	{
		$this->is_modal_active = bt_manager()->get_option( 'donation_form_layout' ) === 'modal' ? true : false;
		
		$this->enqueue_scripts();
		
		extract( array (
				'args' => $args 
		) );
		
		if ( $this->is_modal_active ) {
			
			bfwc_get_template( 'donations/views/modal-form.php', array (
					'shortcode' => $this, 
					'args' => $args 
			) );
		} else {
			
			bfwc_get_template( 'donations/views/inline-form.php', array (
					'shortcode' => $this, 
					'args' => $args 
			) );
		}
	}

	/**
	 * Output the html for the one time donation page.
	 *
	 * @param array $args        	
	 */
	public function output_transaction_page( $args )
	{
		$this->is_transaction = true;
		$this->output( $args );
	}

	/**
	 * Output html for a recurring donation.
	 *
	 * @param array $args        	
	 */
	public function output_recurring_page( $args )
	{
		$this->get_braintree_plans();
		$this->output( $args );
	}

	public function process_donation()
	{
		$enabled_fields = $this->get_enabled_fields();
		
		foreach ( $enabled_fields as $field_name => $field ) {
			$value = apply_filters( "braintree_donation_validate_field_{$field}", $this->get_request_parameter( $field_name ) );
			if ( empty( $value ) ) {
				$this->messages [] = sprintf( __( 'Field %s cannot be blank.', 'braintree' ), $field [ 'label' ] );
			}
		}
		if ( $this->messages ) {
			$this->return_error();
		}
		$attribs = array ();
		
		// Add the billing address to the attribs array.
		$this->add_billing_address( $attribs );
		$this->add_partner_code( $attribs );
		$this->add_merchant_account_id( $attribs );
		$this->add_payment_method( $attribs );
		$this->add_customer( $attribs );
		$this->add_options( $attribs );
		$donation_id = $this->create_donation();
		$attribs [ 'orderId' ] = $donation_id;
		$attribs [ 'amount' ] = $this->get_request_parameter( 'donation_amount' );
		try {
			$result = Braintree_Transaction::sale( $attribs );
			if ( $result->success ) {
				$transaction = $result->transaction;
				$donation = new Braintree_Gateway_Donation( $donation_id );
				if ( $transaction->status === 'submitted_for_settlement' ) {
					$donation->update_status( 'complete' );
				}
				delete_transient( $_COOKIE [ $this->_cookie ] );
				bt_manager()->success( sprintf( __( 'Donation %s has been processed in the amount of %s.', 'braintree' ), $result->transaction->orderId, $result->transaction->amount ) );
				braintree_update_donation( $donation_id, array (
						'transaction_id' => $transaction->id, 
						'amount' => $transaction->amount, 
						'payment_method_token' => $transaction->token, 
						'merchant_account_id' => $transaction->merchantAccountId, 
						'payment_method_title' => braintree_get_payment_method_title_from_transaction( $transaction ) 
				) );
				wp_send_json( array (
						'result' => 'success', 
						'redirect_url' => bt_manager()->get_option( 'donation_success_url' ) 
				) );
				exit();
			} else {
				$this->messages [] = sprintf( __( 'There was an error processing your donation. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) );
				$this->return_error();
			}
		} catch( \Braintree\Exception $e ) {
			bt_manager()->error( sprintf( __( 'Error processing donation %s.', 'braintree' ), $donation_id ) );
			$this->messages [] = sprintf( __( 'There was an error processing your donation. Reason: %s', 'braintree' ), $e->getMessage() );
			$this->return_error();
		}
	}

	/**
	 * Process the Braintree Subscription.
	 */
	public function process_recurring_donation()
	{
		$attribs = array ();
		
		$enabled_fields = $this->get_enabled_fields();
		
		foreach ( $enabled_fields as $field_name => $field ) {
			$value = apply_filters( "braintree_donation_validate_field_{$field}", $this->get_request_parameter( $field_name ) );
			if ( empty( $value ) ) {
				$this->messages [] = sprintf( __( 'Field %s cannot be blank.', 'braintree' ), $field [ 'label' ] );
			}
		}
		if ( $this->messages ) {
			$this->return_error();
		}
		
		$donation_id = $this->create_donation( 'bt_rc_donation' );
		
		$subscription_attribs = array ();
		
		if ( ! is_user_logged_in() ) {
			// If the donor is not logged in then we need to create a Braintree
			// Customer so we can save the payment method and use it for the
			// subscription creation.
			try {
				$result = Braintree_Customer::create( array (
						'firstName' => $this->get_request_parameter( 'billing_first_name' ), 
						'lastName' => $this->get_request_parameter( 'billing_last_name' ), 
						'company' => $this->get_request_parameter( 'billing_company' ), 
						'email' => $this->get_request_parameter( 'email_address' ), 
						'paymentMethodNonce' => $this->get_request_parameter( 'payment_method_nonce' ) 
				) );
				if ( $result->success ) {
					$payment_method = $result->customer->paymentMethods [ 0 ];
					$subscription_attribs [ 'paymentMethodToken' ] = $payment_method->token;
					$payment_method_title = braintree_get_payment_method_title_from_method_details( $payment_method );
				} else {
					$this->messages [] = sprintf( __( 'There was an error creating a user in our system. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) );
				}
			} catch( Exception $e ) {
				$this->messages [] = sprintf( __( 'There was an error creating a user in our system.', 'braintree' ) );
				$this->return_error();
			}
		} else {
			if ( $this->use_saved_method() ) {
				$user_id = wp_get_current_user()->ID;
				$token = $this->get_request_parameter( 'payment_method_token' );
				$subscription_attribs [ 'paymentMethodToken' ] = $token;
				$payment_method_title = braintree_get_payment_method_title_from_array( braintree_get_payment_method_from_token( $user_id, $token ) );
			} else {
				try {
					$result = Braintree_PaymentMethod::create( array (
							'customerId' => bt_manager()->get_customer_id( wp_get_current_user()->ID ), 
							'paymentMethodNonce' => $this->get_request_parameter( 'payment_method_nonce' ) 
					) );
					if ( $result->success ) {
						bt_manager()->save_user_payment_method( wp_get_current_user()->ID, $result->paymentMethod );
						$subscription_attribs [ 'paymentMethodToken' ] = $result->paymentMethod->token;
						$payment_method_title = braintree_get_payment_method_title_from_method_details( $result->paymentMethod );
					} else {
						$this->messages [] = sprintf( __( 'There was an error saving your payment method. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) );
						$this->return_error();
					}
				} catch( Exception $e ) {
					$this->messages [] = sprintf( __( 'An exception was an encountered while saving your payment method.', 'braintree' ) );
					$this->return_error();
				}
			}
		}
		
		$subscription_attribs [ 'planId' ] = $this->get_request_parameter( 'recurring_donaton_plan' );
		$this->add_merchant_account_id( $subscription_attribs );
		$subscription_attribs [ 'id' ] = $donation_id;
		
		try {
			$result = Braintree_Subscription::create( $subscription_attribs );
			
			if ( $result->success ) {
				delete_transient( $_COOKIE [ $this->_cookie ] );
				$subscription = $result->subscription;
				braintree_update_donation( $donation_id, array (
						'merchant_account_id' => $subscription->merchantAccountId, 
						'first_billing_date' => $subscription->firstBillingDate, 
						'plan_id' => $subscription->planId, 
						'amount' => $subscription->price, 
						'recurring' => true, 
						'payment_method_title' => $payment_method_title 
				) );
				wp_send_json( array (
						'result' => 'success', 
						'redirect_url' => bt_manager()->get_option( 'donation_success_url' ) 
				) );
				exit();
			} else {
				$this->messages [] = sprintf( __( 'There was an error creating your recurring donation. Reason: %s', 'braintree' ), bfwc_get_error_message( $result ) );
				$this->return_error();
			}
		} catch( Exception $e ) {
			$this->messages [] = __( 'There was an error creating your recurring donation.', 'braintree' );
			$this->return_error();
		}
	}

	/**
	 * Create a donation or retrieve an existing donation.
	 *
	 * @return int $donation_id
	 */
	public function create_donation( $type = 'braintree_donation' )
	{
		$args = array ();
		$transient = $_COOKIE [ $this->_cookie ];
		$donation_id = get_transient( $transient );
		foreach ( $this->get_enabled_fields() as $field_name => $field ) {
			$args [ $field_name ] = $this->get_request_parameter( $field_name );
		}
		$args [ 'user_id' ] = is_user_logged_in() ? wp_get_current_user()->ID : 0;
		$args [ 'currency' ] = bt_manager()->get_option( 'donation_currency' );
		if ( ! $donation_id ) {
			switch( $type) {
				case 'braintree_donation' :
					$donation_id = braintree_create_donation( $args );
					break;
				case 'bt_rc_donation' :
					$donation_id = braintree_create_recurring_donation( $args );
					break;
			}
			set_transient( $transient, $donation_id, 0 );
		} else {
			braintree_update_donation( $donation_id, $args );
			
			// always update the post type in case customer has switched from
			// donation to recurring donation or vice versa.
			wp_update_post( array (
					'post_type' => $type, 
					'ID' => $donation_id 
			) );
		}
		return $donation_id;
	}

	private function return_error()
	{
		return wp_send_json( array (
				'result' => 'failure', 
				'messages' => $this->messages 
		) );
		exit();
	}

	/**
	 * Enqueue all scripts that have been registered.
	 */
	public function enqueue_scripts()
	{
		foreach ( $this->scripts [ 'js' ] as $script ) {
			wp_enqueue_script( $script );
		}
		foreach ( $this->scripts [ 'css' ] as $style ) {
			wp_enqueue_style( $style );
		}
		wp_localize_script( 'braintree-donation-js', 'braintree_donation_vars', $this->localized_vars() );
		wp_localize_script( 'braintree-donation-js', 'braintree_error_messages', bfwc_get_combined_error_messages() );
		wp_localize_script( 'braintree-donation-js', 'bfwc_field_vars', self::get_field_vars() );
	}

	/**
	 * Enqueue all scripts needed for the donation functionality to work.
	 */
	public function register_scripts()
	{
		wp_register_script( 'braintree-donation-js', bt_manager()->plugin_assets_path() . 'js/frontend/donations.js', array (
				'jquery' 
		), bt_manager()->version, true );
		
		wp_register_script( 'braintree-jquery-block', bt_manager()->plugin_assets_path() . 'js/jquery/blockUI.min.js', array (
				'jquery' 
		), bt_manager()->version, true );
		wp_register_script( 'braintree-select2', bt_manager()->plugin_assets_path() . 'js/select2/select2.min.js', array (
				'braintree-donation-js' 
		), bt_manager()->version, true );
		
		wp_register_style( 'braintree-donations-css', bt_manager()->plugin_assets_path() . 'css/donations.css', null, bt_manager()->version, null );
		wp_register_style( 'braintree-select2-style', bt_manager()->plugin_assets_path() . 'css/select2.css', null, bt_manager()->version, null );
		
		$this->scripts [ 'js' ] [] = 'braintree-donation-js';
		$this->scripts [ 'js' ] [] = 'braintree-jquery-block';
		$this->scripts [ 'js' ] [] = 'braintree-message-handler';
		$this->scripts [ 'js' ] [] = 'braintree-select2';
		$this->scripts [ 'css' ] [] = 'braintree-donations-css';
		$this->scripts [ 'css' ] [] = 'braintree-styles';
		$this->scripts [ 'css' ] [] = 'braintree-select2-style';
		
		if ( bt_manager()->get_option( 'donation_form_type' ) === 'dropin' ) {
			
			wp_register_script( 'braintree-donation-dropin-js', BRAINTREE_JS_DROPIN, array (
					'jquery' 
			), bt_manager()->version, true );
			
			$this->scripts [ 'js' ] [] = 'braintree-donation-dropin-js';
		} else {
			
			wp_register_script( 'braintree-donation-client-js', BRAINTREE_JS_V3_CLIENT, array (), bt_manager()->version, true );
			
			wp_register_script( 'braintree-donation-hosted-fields-js', BRAINTREE_JS_V3_HOSTED, array (), bt_manager()->version, true );
			
			wp_register_style( 'braintree-custom-form-css', $this->get_custom_form() [ 'css' ], null, bt_manager()->version, null );
			
			if ( ! empty( $this->get_custom_form() [ 'external_css' ] ) ) {
				wp_register_style( 'braintree-custom-form-external-css', $this->get_custom_form() [ 'external_css' ], null, bt_manager()->version, null );
			}
			if ( ! empty( $this->get_custom_form() [ 'javascript' ] ) ) {
				wp_register_script( 'braintree-donation-external-js', $this->get_custom_form() [ 'javascript' ], array (
						'braintree-donation-js' 
				), bt_manager()->version, true );
				$this->scripts [ 'js' ] [] = 'braintree-donation-external-js';
			}
			$this->scripts [ 'js' ] [] = 'braintree-donation-client-js';
			$this->scripts [ 'js' ] [] = 'braintree-donation-hosted-fields-js';
			$this->scripts [ 'css' ] [] = 'braintree-custom-form-css';
			$this->scripts [ 'css' ] [] = 'braintree-custom-form-external-css';
		}
	}

	public function localized_vars()
	{
		$vars = array (
				'client_token' => bt_manager()->get_client_token(), 
				'dropin_form' => array (
						'enabled' => bt_manager()->get_option( 'donation_form_type' ) === 'dropin' ? true : false 
				) 
		);
		if ( $this->is_transaction ) {
			$vars [ 'ajax_url' ] = admin_url() . 'admin-ajax.php?action=' . $this->ajax_action;
		} else {
			$vars [ 'ajax_url' ] = admin_url() . 'admin-ajax.php?action=' . $this->ajax_recurring_action;
		}
		if ( $this->is_modal_active ) {
			$vars [ 'modal' ] = true;
		}
		
		if ( ! $vars [ 'dropin_form' ] [ 'enabled' ] ) {
			$vars [ 'custom_form' ] = array (
					'styles' => json_decode( bt_manager()->get_option( 'donation_custom_form_styles' ), true ), 
					'fields' => $this->get_custom_form_fields() 
			);
		}
		$vars [ 'loader' ] = array (
				'loader_enabled' => bt_manager()->is_active( 'donation_enable_loader' ), 
				'loader_html' => bt_manager()->get_option( 'donation_custom_form_loader_message' ), 
				'loader_css' => bt_manager()->get_option( 'donation_custom_form_loader_css' ) 
		);
		
		return $vars;
	}

	public function get_donation_fields()
	{
		return apply_filters( 'braintree_get_donation_fields', array (
				'billing_first_name' => array (
						'type' => 'text', 
						'label' => __( 'First Name', 'braintree' ), 
						'class' => '', 
						'placeholder' => __( 'First Name', 'braintree' ), 
						'attributes' => array (
								'data-bt-donation' => 'billing_first_name' 
						), 
						'required' => true 
				), 
				'billing_last_name' => array (
						'type' => 'text', 
						'label' => __( 'Last Name', 'braintree' ), 
						'class' => '', 
						'placeholder' => __( 'Last Name', 'braintree' ), 
						'attributes' => array (
								'data-bt-donation' => 'billing_last_name' 
						), 
						'required' => true 
				), 
				'billing_company' => array (
						'type' => 'text', 
						'label' => __( 'Company Name', 'braintree' ), 
						'class' => '', 
						'placeholder' => __( 'Company Name', 'braintree' ), 
						'attributes' => array (
								'data-bt-donation' => 'billing_company' 
						), 
						'required' => true 
				), 
				'billing_address_1' => array (
						'type' => 'text', 
						'label' => __( 'Billing Address', 'braintree' ), 
						'placeholder' => __( 'Address', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'billing_address_1' 
						), 
						'required' => true 
				), 
				'billing_address_2' => array (
						'type' => 'text', 
						'label' => __( 'Billing Address 2', 'billing_address_2' ), 
						'placeholder' => __( 'Address2', 'data-bt-donation' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-stripe' => 'billing_address_2' 
						), 
						'required' => false 
				), 
				'billing_city' => array (
						'type' => 'text', 
						'label' => __( 'Billing City', 'braintree' ), 
						'placeholder' => __( 'City', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'address_city' 
						), 
						'required' => true 
				), 
				'billing_country' => array (
						'type' => 'select', 
						'label' => __( 'Country', 'braintree' ), 
						'placeholder' => __( 'Country', 'braintree' ), 
						'options' => braintree_get_countries(), 
						'class' => 'bfwc-select2 billing-country', 
						'value' => 'US', 
						'attributes' => array (
								'data-bt-donation' => 'address_country' 
						), 
						'required' => true 
				), 
				'billing_state' => array (
						'type' => 'text', 
						'label' => __( 'State / Region', 'braintree' ), 
						'placeholder' => __( 'State', 'braintree' ), 
						'class' => 'billing-state', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'billing_state' 
						), 
						'required' => false 
				), 
				'billing_postalcode' => array (
						'type' => 'text', 
						'label' => __( 'Postal Code', 'braintree' ), 
						'placeholder' => __( 'Postal Code', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'billing_postalcode' 
						), 
						'required' => true 
				), 
				'email_address' => array (
						'type' => 'text', 
						'label' => __( 'Email Address', 'braintree' ), 
						'placeholder' => __( 'Email Address', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'email_address' 
						), 
						'required' => true 
				), 
				'donation_message' => array (
						'type' => 'textarea', 
						'label' => __( 'Donation Message', 'braintree' ), 
						'placeholder' => __( 'Donation Message', 'braintree' ), 
						'class' => '', 
						'value' => '', 
						'attributes' => array (
								'data-bt-donation' => 'donation_message' 
						), 
						'required' => false 
				) 
		) );
	}

	public function get_custom_form()
	{
		$form_name = bt_manager()->get_option( 'donation_custom_form_design' );
		return braintree_get_custom_donation_forms() [ $form_name ];
	}

	public function get_custom_form_fields()
	{
		return $fields = apply_filters( 'braintre_donations_custom_fields', array (
				'number' => array (
						'selector' => '#bfwc-card-number', 
						'placeholder' => bt_manager()->get_option( 'donation_card_number_placeholder' ) 
				), 
				'cvv' => array (
						'selector' => '#bfwc-cvv', 
						'placeholder' => bt_manager()->get_option( 'donation_card_cvv_placeholder' ) 
				), 
				'postalCode' => array (
						'selector' => '#bfwc-postal-code', 
						'placeholder' => bt_manager()->get_option( 'donation_card_postal_placeholder' ) 
				), 
				'expirationDate' => array (
						'selector' => '#bfwc-expiration-date', 
						'placeholder' => bt_manager()->get_option( 'donation_card_expiration_date_placeholder' ) 
				), 
				'expirationMonth' => array (
						'selector' => '#bfwc-expiration-month', 
						'placeholder' => bt_manager()->get_option( 'donation_card_expiration_month_placeholder' ) 
				), 
				'expirationYear' => array (
						'selector' => '#bfwc-expiration-year', 
						'placeholder' => bt_manager()->get_option( 'donation_card_expiration_year_placeholder' ) 
				) 
		) );
	}

	public function get_button_styles()
	{
		$styles = apply_filters( 'braintree_donation_button_styles', array (
				'background-color' => bt_manager()->get_option( 'donation_button_background' ), 
				'border-color' => bt_manager()->get_option( 'donation_button_border' ), 
				'color' => bt_manager()->get_option( 'donation_button_text_color' ) 
		) );
		$css = array ();
		
		foreach ( $styles as $style => $value ) {
			$css [] = $style . ':' . $value . ';';
		}
		return implode( ' ', $css );
	}

	public function get_modal_button_styles()
	{
		$styles = apply_filters( 'braintree_modal_button_styles', array (
				'background-color' => bt_manager()->get_option( 'donation_modal_button_background' ), 
				'border-color' => bt_manager()->get_option( 'donation_modal_button_border' ), 
				'color' => bt_manager()->get_option( 'donation_modal_button_text_color' ) 
		) );
		$css = array ();
		
		foreach ( $styles as $style => $value ) {
			$css [] = $style . ':' . $value . ';';
		}
		return implode( ' ', $css );
	}

	private function get_request_parameter( $name )
	{
		return isset( $_POST [ $name ] ) ? $_POST [ $name ] : '';
	}

	public function add_billing_address( &$attribs = array() )
	{
		$attribs [ 'billing' ] = array (
				'firstName' => $this->get_request_parameter( 'billing_first_name' ), 
				'lastName' => $this->get_request_parameter( 'billing_last_name' ), 
				'locality' => $this->get_request_parameter( 'billing_city' ), 
				'countryCodeAlpha2' => $this->get_request_parameter( 'billing_country' ), 
				'postalCode' => $this->get_request_parameter( 'billing_postalcode' ), 
				'region' => $this->get_request_parameter( 'billing_state' ), 
				'streetAddress' => $this->get_request_parameter( 'billing_address_1' ), 
				'extendedAddress' => $this->get_request_parameter( 'billing_address_2' ) 
		);
	}

	public function add_partner_code( &$attribs = array() )
	{
		$attribs [ 'channel' ] = bt_manager()->get_partner_code();
	}

	public function add_merchant_account_id( &$attribs = array() )
	{
		$merchant_account = bt_manager()->get_option( 'donation_' . bt_manager()->get_environment() . '_merchant_account_id' );
		if ( ! empty( $merchant_accouts ) ) {
			$attribs [ 'merchantAccountId' ] = $merchant_account;
		}
	}

	public function add_payment_method( &$attribs )
	{
		if ( $this->use_saved_method() ) {
			$attribs [ 'paymentMethodToken' ] = $this->get_request_parameter( 'payment_method_token' );
		} else {
			$attribs [ 'paymentMethodNonce' ] = $this->get_request_parameter( 'payment_method_nonce' );
		}
	}

	public function add_customer( &$attribs )
	{
		$attribs [ 'customer' ] = array (
				'firstName' => $this->get_request_parameter( 'billing_first_name' ), 
				'lastName' => $this->get_request_parameter( 'billing_last_name' ) 
		);
		$email = $this->get_request_parameter( 'email_address' );
		if ( ! empty( $email ) ) {
			$attribs [ 'customer' ] [ 'email' ] = $this->get_request_parameter( 'email_address' );
		}
	}

	public function add_options( &$attribs )
	{
		$attribs [ 'options' ] = array (
				'submitForSettlement' => ! bt_manager()->is_active( 'authorize_donation' ) 
		);
		
		$attribs [ 'taxExempt' ] = bt_manager()->is_active( 'donation_tax_exempt' );
	}

	/**
	 * Get an existing donation cookie or create a new one.
	 * The cookie value points to a transient value in the database that
	 * contains an unfinished donation.
	 */
	public function init_cookie()
	{
		if ( braintree_is_donation_page() ) {
			if ( isset( $_COOKIE [ $this->_cookie ] ) ) {
				$this->donation = get_transient( $_COOKIE [ $this->_cookie ] );
			} else {
				setcookie( $this->_cookie, 'braintree_donation_' . md5( random_bytes( 32 ) ), 0, COOKIEPATH ? COOKIEPATH : '/', COOKIE_DOMAIN, is_ssl() ? true : false );
			}
		}
	}

	public function get_enabled_fields()
	{
		$fields = array ();
		$enabled_fields = bt_manager()->get_option( 'donation_fields' );
		if ( ! empty( $enabled_fields ) ) {
			foreach ( $enabled_fields as $k ) {
				$fields [ $k ] = $this->get_donation_fields() [ $k ];
			}
		}
		return $fields;
	}

	public function get_amount_input_field( $attrs )
	{
		if ( empty( $attrs ) ) {
			$input = array (
					'type' => 'text', 
					'label' => __( 'Donation Amount', 'braintree' ), 
					'placeholder' => __( 'Amount', 'braintree' ), 
					'value' => '', 
					'class' => '', 
					'required' => true, 
					'attributes' => array () 
			);
		} else {
			$input = array (
					'type' => 'select', 
					'label' => __( 'Donation Amount', 'braintree' ), 
					'placeholder' => __( 'Amount', 'braintree' ), 
					'value' => '', 
					'class' => 'bfwc-select2', 
					'required' => 'bfwc-select2', 
					'attributes' => array () 
			);
			foreach ( $attrs as $i => $amount ) {
				$currency_symbol = braintree_get_currency_symbol( bt_manager()->get_option( 'donation_currency' ) );
				$input [ 'options' ] [ $amount ] = sprintf( __( '%s %s' ), $currency_symbol, $amount );
			}
		}
		return $input;
	}

	public function get_recurring_plan_field( $attrs )
	{
		global $plans;
		if ( isset( $attrs [ 'plan_id' ] ) ) {
		
		} else {
			$input = array (
					'type' => 'select', 
					'label' => __( 'Recurring Donation', 'braintree' ), 
					'placeholder' => __( 'Donation', 'braintree' ), 
					'value' => '', 
					'class' => 'bfwc-select2', 
					'required' => true, 
					'attributes' => array () 
			);
			foreach ( $plans as $plan ) {
				if ( in_array( $plan->id, $attrs ) ) {
					$input [ 'options' ] [ $plan->id ] = $plan->description;
				}
			}
		}
		return $input;
	}

	public function get_braintree_plans()
	{
		global $plans;
		try {
			$plans = Braintree_Plan::all();
		} catch( Exception $e ) {
			bt_manager()->error( sprintf( __( 'There was exception thrown while retrieving the Braintree Plans required for a recurring donation.', 'braintree' ) ) );
		}
		return $plans;
	}

	public function use_saved_method()
	{
		$saved_token = $this->get_request_parameter( 'payment_method_token' );
		return ! empty( $saved_token );
	}

	public static function get_field_vars()
	{
		$vars = array (
				'states' => array (
						'US' => braintree_get_states() 
				) 
		);
		return apply_filters( 'bfwc_donation_localized_field_vars', $vars );
	}

}
new Braintree_Gateway_Donation_Shortcode();